from py42.clients import BaseClient


class StorageArchiveClient(BaseClient):
    def search_paths(
        self,
        session_id,
        device_guid,
        regex=None,
        max_results=None,
        timestamp=None,
        show_deleted=None,
    ):
        # session_id is a web restore session ID (see create_restore_session)
        uri = u"/api/WebRestoreSearch"
        params = {
            u"webRestoreSessionId": session_id,
            u"guid": device_guid,
            u"regex": regex,
            u"maxResults": max_results,
            u"timestamp": timestamp,
            u"showDeleted": show_deleted,
        }
        return self._session.get(uri, params=params)

    def get_file_size(
        self, device_guid, file_id, timestamp=None, show_deleted=None, backup_set_id=None
    ):
        uri = u"/api/WebRestoreFileSize"
        params = {
            u"guid": device_guid,
            u"fileId": file_id,
            u"timestamp": timestamp,
            u"showDeleted": show_deleted,
            u"backupSetId": backup_set_id,
        }
        return self._session.get(uri, params=params)

    def get_file_path_metadata(
        self,
        session_id,
        device_guid,
        file_id=None,
        timestamp=None,
        show_deleted=None,
        batch_size=None,
        last_batch_file_id=None,
        backup_set_id=None,
        include_os_metadata=None,
    ):
        # session_id is a web restore session ID (see create_restore_session)
        uri = u"/api/WebRestoreTreeNode"
        params = {
            u"webRestoreSessionId": session_id,
            u"guid": device_guid,
            u"fileId": file_id,
            u"timestamp": timestamp,
            u"showDeleted": show_deleted,
            u"batchSize": batch_size,
            u"lastBatchFileId": last_batch_file_id,
            u"backupSetId": backup_set_id,
            u"includeOsMetadata": include_os_metadata,
        }
        return self._session.get(uri, params=params)

    def create_restore_session(
        self, device_guid, data_key_token=None, private_password=None, encryption_key=None
    ):
        """Creates a web restore session.
        See https://console.us.code42.com/apidocviewer/#WebRestoreSession
        """
        uri = u"/api/WebRestoreSession"
        json_dict = {
            u"computerGuid": device_guid,
            u"dataKeyToken": data_key_token,
            u"privatePassword": private_password,
            u"encryptionKey": encryption_key,
        }
        return self._session.post(uri, json=json_dict)

    def start_restore(
        self,
        guid,
        web_restore_session_id,
        path_set,
        num_files,
        num_dirs,
        size,
        zip_result=None,
        expire_job=None,
        show_deleted=None,
        restore_full_path=None,
        timestamp=None,
        exceptions=None,
        backup_set_id=None,
    ):
        """Submits a web restore job.
        See https://console.us.code42.com/apidocviewer/#WebRestoreJob-post
        """
        uri = u"/api/WebRestoreJob"
        json_dict = {
            u"guid": guid,
            u"webRestoreSessionId": web_restore_session_id,
            u"pathSet": path_set,
            u"numFiles": num_files,
            u"numDirs": num_dirs,
            u"size": size,
            u"zipResult": zip_result,
            u"expireJob": expire_job,
            u"showDeleted": show_deleted,
            u"restoreFullPath": restore_full_path,
            u"timestamp": timestamp,
            u"exceptions": exceptions,
            u"backupSetId": backup_set_id,
        }

        return self._session.post(uri, json=json_dict)

    def get_restore_status(self, job_id):
        uri = u"/api/WebRestoreJob/{}".format(job_id)
        return self._session.get(uri)

    def cancel_restore(self, job_id):
        uri = u"/api/WebRestoreJob"
        json_dict = {u"jobId": job_id}
        return self._session.delete(uri, json=json_dict)

    def stream_restore_result(self, job_id):
        uri = u"/api/WebRestoreJobResult/{}".format(job_id)
        return self._session.get(uri, stream=True)



from py42._internal.auth_handling import AuthHandler, HeaderModifier
from py42._internal.session import Py42Session
from py42._internal.token_providers import (
    BasicAuthProvider,
    C42ApiV1TokenProvider,
    C42ApiV3TokenProvider,
)


class SessionFactory(object):
    def __init__(self, session_impl, session_modifier_factory, auth_handler_factory):
        self._session_impl = session_impl
        self._session_modifier_factory = session_modifier_factory
        self._auth_handler_factory = auth_handler_factory

    def create_basic_auth_session(self, host_address, username, password):
        provider = BasicAuthProvider(username, password)
        header_modifier = self._session_modifier_factory.create_header_modifier(u"Basic {0}")
        return self._create_session(self._session_impl, host_address, provider, header_modifier)

    def create_v1_session(self, host_address, parent_session):
        provider = C42ApiV1TokenProvider(parent_session)
        header_modifier = self._session_modifier_factory.create_header_modifier(u"token {0}")
        return self._create_session(self._session_impl, host_address, provider, header_modifier)

    def create_jwt_session(self, host_address, parent_session):
        provider = C42ApiV3TokenProvider(parent_session)
        header_modifier = self._session_modifier_factory.create_header_modifier(
            u"v3_user_token {0}"
        )
        return self._create_session(self._session_impl, host_address, provider, header_modifier)

    def create_storage_session(self, host_address, c42_api_token_provider):
        header_modifier = self._session_modifier_factory.create_header_modifier(u"login_token {0}")
        tmp = self._create_session(
            self._session_impl, host_address, c42_api_token_provider, header_modifier
        )
        return self.create_v1_session(host_address, tmp)

    def create_anonymous_session(self, host_address):
        return self._create_session(self._session_impl, host_address)

    def _create_session(self, session_impl, host_address, token_provider=None, modifier=None):
        handler = None
        if modifier:
            handler = self._auth_handler_factory.create_auth_handler(token_provider, modifier)
        return Py42Session(session_impl(), host_address, auth_handler=handler)


class SessionModifierFactory(object):
    @staticmethod
    def create_header_modifier(value_format):
        return HeaderModifier(value_format=value_format)


class AuthHandlerFactory(object):
    @staticmethod
    def create_auth_handler(token_provider, modifier=None):
        return AuthHandler(token_provider, modifier)



from py42.sdk.queries.fileevents.file_event_query import (
    FileEventFilterStringField,
    FileEventFilterComparableField,
)
from py42.sdk.queries.query_filter import QueryFilterStringField


class FileCategory(QueryFilterStringField):
    """Class that filters events by category of the file observed."""

    _term = u"fileCategory"


class FileName(FileEventFilterStringField):
    """Class that filters events by the name of the file observed."""

    _term = u"fileName"


class FileOwner(FileEventFilterStringField):
    """Class that filters events by the owner of the file observed."""

    _term = u"fileOwner"


class FilePath(FileEventFilterStringField):
    """Class that filters events by path of the file observed."""

    _term = u"filePath"


class FileSize(FileEventFilterComparableField):
    """Class that filters events by size of the file observed.

    Size ``value`` must be bytes.
    """

    _term = u"fileSize"


class MD5(FileEventFilterStringField):
    """Class that filters events by the MD5 hash of the file observed."""

    _term = u"md5Checksum"


class SHA256(FileEventFilterStringField):
    """Class that filters events by SHA256 hash of the file observed."""

    _term = u"sha256Checksum"



from py42.clients import BaseClient


class AdministrationClient(BaseClient):
    def get_diagnostics(self, include_volumes=None):
        uri = u"/api/Diagnostic"
        params = {u"incVolumes": include_volumes}
        return self._session.get(uri, params=params)

    def get_alert_log(self, status=None, alert_type=None, page_num=None, page_size=None):
        uri = u"/api/AlertLog"
        params = {u"status": status, u"type": alert_type, u"pgNum": page_num, u"pgSize": page_size}
        return self._session.get(uri, params=params)

    def get_current_tenant(self):
        uri = u"/c42api/v3/customer/my"
        return self._session.get(uri)



import json

from py42.clients import BaseClient
from py42.clients.util import get_all_pages
from py42.exceptions import Py42NotFoundError, Py42BadRequestError


class HighRiskEmployeeClient(BaseClient):
    """Administrator utility to manage High Risk employees.

    `Support Documentation <https://support.code42.com/Administrator/Cloud/Monitoring_and_managing/Detection_list_management_APIs>`__

    """

    _api_version = u"v2"
    _uri_prefix = u"/svc/api/{0}".format(_api_version)
    _resource = u"/highriskemployee"

    def __init__(self, session, user_context, detection_list_user_client, user_client):
        super(HighRiskEmployeeClient, self).__init__(session)
        self._user_context = user_context
        self._user_client = user_client
        self._detection_list_user_client = detection_list_user_client

    def _make_uri(self, action):
        return u"{0}{1}{2}".format(self._uri_prefix, self._resource, action)

    def _add_high_risk_employee(self, tenant_id, user_id):

        data = {u"tenantId": tenant_id, u"userId": user_id}
        uri = self._make_uri(u"/add")
        return self._session.post(uri, data=json.dumps(data))

    def add(self, user_id):
        """Adds a user to high risk employee detection list.

        A user profile in detection list will be created before adding to High Risk Employee
        list if it doesn't exist.

        Returns failure when a user already exists in the high risk employee detection list.

        Args:
            user_id (str or int): The Code42 userUid of the user who needs to be added to HRE detection list.

        Returns:
            :class:`py42.response.Py42Response`
        """
        try:
            self._detection_list_user_client.get_by_id(user_id)
        except (Py42NotFoundError, Py42BadRequestError):
            user = self._user_client.get_by_uid(user_id)
            self._detection_list_user_client.create(user[u"username"])

        tenant_id = self._user_context.get_current_tenant_id()
        return self._add_high_risk_employee(tenant_id, user_id)

    def set_alerts_enabled(self, enabled=True):
        """Enable alerts.

        Args:
            enabled (bool): Whether to enable alerts for all users

        Returns:
            :class:`py42.response.Py42Response`
        """
        data = {u"tenantId": self._user_context.get_current_tenant_id(), u"alertsEnabled": enabled}
        uri = self._make_uri(u"/setalertstate")
        return self._session.post(uri, data=json.dumps(data))

    def remove(self, user_id):
        """Remove a user from high risk employee detection list.

        Args:
            user_id (str or int): The Code42 userUid of the user who needs to be added to HRE detection lists.

        Returns:
            :class:`py42.response.Py42Response`
        """
        data = {u"tenantId": self._user_context.get_current_tenant_id(), u"userId": user_id}
        uri = self._make_uri(u"/remove")
        return self._session.post(uri, data=json.dumps(data))

    def get(self, user_id):
        """Get user information.

        Args:
            user_id (str or int): The Code42 userUid of the user has been added to the High Risk Employee
              detection list.

        Returns:
            :class:`py42.response.Py42Response`
        """
        data = {u"tenantId": self._user_context.get_current_tenant_id(), u"userId": user_id}
        uri = self._make_uri(u"/get")
        return self._session.post(uri, data=json.dumps(data))

    def _get_high_risk_employees_page(
        self,
        tenant_id,
        filter_type=None,
        sort_key=None,
        sort_direction=None,
        page_num=None,
        page_size=None,
    ):
        # Overwriting page_size since default value 1000 returns error
        page_size = 100
        data = {
            u"tenantId": tenant_id,
            u"filterType": filter_type,
            u"pgNum": page_num,
            u"pgSize": page_size,
            u"srtKey": sort_key,
            u"srtDirection": sort_direction,
        }

        uri = self._make_uri(u"/search")
        return self._session.post(uri, data=json.dumps(data))

    def get_all(self, filter_type=u"OPEN", sort_key=None, sort_direction=None):
        """Search High Risk employee list. Filter results by filter_type.

        Args:
            filter_type (str): Valid filter types.
            sort_key (str): Sort results based by field.
            sort_direction (str): "ASC" or "DESC"

        Returns:
            generator: An object that iterates over :class:`py42.response.Py42Response` objects
            that each contain a page of users.
        """

        return get_all_pages(
            self._get_high_risk_employees_page,
            u"items",
            tenant_id=self._user_context.get_current_tenant_id(),
            filter_type=filter_type,
            sort_key=sort_key,
            sort_direction=sort_direction,
        )



import posixpath
import time
from collections import namedtuple

from py42.exceptions import Py42ArchiveFileNotFoundError

FileSelection = namedtuple(u"FileSelection", u"path_set, num_files, num_dirs, size")


class FileType(object):
    DIRECTORY = u"directory"
    FILE = u"file"


class ArchiveAccessorManager(object):
    def __init__(self, archive_client, storage_client_factory):
        self._archive_client = archive_client
        self._storage_client_factory = storage_client_factory

    def get_archive_accessor(
        self, device_guid, destination_guid=None, private_password=None, encryption_key=None
    ):
        client = self._storage_client_factory.from_device_guid(
            device_guid, destination_guid=destination_guid
        )
        decryption_keys = self._get_decryption_keys(device_guid, private_password, encryption_key)
        session_id = self._create_restore_session(client.archive, device_guid, **decryption_keys)
        restore_job_manager = create_restore_job_manager(client.archive, device_guid, session_id)
        return ArchiveAccessor(device_guid, session_id, client.archive, restore_job_manager)

    def _get_decryption_keys(self, device_guid, private_password, encryption_key):
        decryption_keys = {}
        if encryption_key:
            decryption_keys["encryption_key"] = encryption_key
        else:
            data_key_token = self._get_data_key_token(device_guid) if not encryption_key else None
            if data_key_token:
                decryption_keys["data_key_token"] = data_key_token

            if private_password:
                decryption_keys["private_password"] = private_password
        return decryption_keys

    def _get_data_key_token(self, device_guid):
        return self._archive_client.get_data_key_token(device_guid)[u"dataKeyToken"]

    @staticmethod
    def _create_restore_session(storage_archive_client, device_guid, **kwargs):
        response = storage_archive_client.create_restore_session(device_guid, **kwargs)
        return response[u"webRestoreSessionId"]


class ArchiveAccessor(object):

    DEFAULT_DIRECTORY_DOWNLOAD_NAME = u"download"
    JOB_POLLING_INTERVAL = 1

    def __init__(
        self, device_guid, archive_session_id, storage_archive_client, restore_job_manager
    ):
        self._device_guid = device_guid
        self._archive_session_id = archive_session_id
        self._storage_archive_client = storage_archive_client
        self._restore_job_manager = restore_job_manager

    def stream_from_backup(self, file_path):
        metadata = self._get_file_via_walking_tree(file_path)
        file_selection = self._build_file_selection(metadata[u"path"], metadata[u"type"])
        return self._restore_job_manager.get_stream(file_selection)

    def _get_file_via_walking_tree(self, file_path):
        path_parts = file_path.split(u"/")
        path_root = path_parts[0] + u"/"

        response = self._get_children(node_id=None)
        for root in response:
            if root[u"path"].lower() == path_root.lower():
                return self._walk_tree(root, path_parts[1:])

        raise Py42ArchiveFileNotFoundError(self._device_guid, file_path)

    def _walk_tree(self, current_node, remaining_path_components):
        if not remaining_path_components or not remaining_path_components[0]:
            return current_node

        children = self._get_children(node_id=current_node[u"id"])
        current_node_path = current_node[u"path"]
        target_child_path = posixpath.join(current_node_path, remaining_path_components[0])

        for child in children:
            if child[u"path"].lower() == target_child_path.lower():
                return self._walk_tree(child, remaining_path_components[1:])

        raise Py42ArchiveFileNotFoundError(self._device_guid, target_child_path)

    def _get_children(self, node_id=None):
        return self._storage_archive_client.get_file_path_metadata(
            self._archive_session_id, self._device_guid, file_id=node_id, show_deleted=True
        )

    @staticmethod
    def _build_file_selection(file_path, file_type):
        path_set = [{u"type": file_type, u"path": file_path, u"selected": True}]
        # pass in dummy values; only used with progress indication, which we don't currently use
        num_files = num_dirs = size = 1
        return FileSelection(path_set, num_files, num_dirs, size)


class RestoreJobManager(object):
    JOB_POLLING_INTERVAL_SECONDS = 1

    def __init__(
        self,
        storage_archive_client,
        device_guid,
        archive_session_id,
        job_polling_interval=JOB_POLLING_INTERVAL_SECONDS,
    ):
        self._storage_archive_client = storage_archive_client
        self._device_guid = device_guid
        self._archive_session_id = archive_session_id
        self._job_polling_interval = job_polling_interval

    def get_stream(self, file_selection):
        response = self._start_restore(file_selection)
        job_id = response["jobId"]

        while not self.is_job_complete(job_id):
            time.sleep(self._job_polling_interval)

        return self._get_stream(job_id)

    def is_job_complete(self, job_id):
        response = self._storage_archive_client.get_restore_status(job_id)
        return self._get_completion_status(response)

    def _start_restore(self, file_selection):
        return self._storage_archive_client.start_restore(
            self._device_guid,
            self._archive_session_id,
            file_selection.path_set,
            file_selection.num_files,
            file_selection.num_dirs,
            file_selection.size,
            show_deleted=True,
        )

    @staticmethod
    def _get_completion_status(response):
        return response[u"done"]

    def _get_stream(self, job_id):
        response = self._storage_archive_client.stream_restore_result(job_id)
        return response


def create_restore_job_manager(storage_archive_client, device_guid, archive_session_id):
    return RestoreJobManager(storage_archive_client, device_guid, archive_session_id)



from requests import Session

from py42._internal.initialization import SDKDependencies
from py42._internal.session_factory import (
    AuthHandlerFactory,
    SessionFactory,
    SessionModifierFactory,
)


def from_local_account(host_address, username, password):
    """Creates a :class:`~py42.sdk.SDKClient` object for accessing the Code42 REST APIs using the
    supplied credentials. Currently, only accounts created within the Code42 console or using the
    APIs (including py42) are supported. Username/passwords that are based on Active Directory,
    Okta, or other Identity providers cannot be used with this method.

    Args:
        host_address (str): The domain name of the Code42 instance being authenticated to, e.g.
            console.us.code42.com
        username (str): The username of the authenticating account.
        password (str): The password of the authenticating account.

    Returns:
        :class:`py42.sdk.SDKClient`
    """
    return SDKClient.from_local_account(host_address, username, password)


class SDKClient(object):
    def __init__(self, sdk_dependencies):
        self._sdk_dependencies = sdk_dependencies

    @classmethod
    def from_local_account(cls, host_address, username, password):
        """Creates a :class:`~py42.sdk.SDKClient` object for accessing the Code42 REST APIs using
        the supplied credentials. Currently, only accounts created within the Code42 console or
        using the APIs (including py42) are supported. Username/passwords that are based on Active
        Directory, Okta, or other Identity providers cannot be used with this method.

        Args:
            host_address (str): The domain name of the Code42 instance being authenticated to, e.g.
                console.us.code42.com
            username (str): The username of the authenticating account.
            password (str): The password of the authenticating account.

        Returns:
            :class:`py42.sdk.SDKClient`
        """
        session_impl = Session
        session_factory = SessionFactory(
            session_impl, SessionModifierFactory(), AuthHandlerFactory()
        )
        basic_auth_session = session_factory.create_basic_auth_session(
            host_address, username, password
        )
        sdk_dependencies = SDKDependencies(host_address, session_factory, basic_auth_session)
        return cls(sdk_dependencies)

    @property
    def serveradmin(self):
        """A collection of methods for getting server information for on-premise environments
        and tenant information for cloud environments.

        Returns:
            :class:`py42.clients.administration.AdministrationClient`
        """
        return self._sdk_dependencies.administration_client

    @property
    def archive(self):
        """A collection of methods for accessing Code42 storage archives. Useful for doing
        web-restores or finding a file on an archive.

        Returns:
            :class:`py42.modules.archive.ArchiveModule`
        """
        return self._sdk_dependencies.archive_module

    @property
    def users(self):
        """A collection of methods for retrieving or updating data about users in the Code42
        environment.

        Returns:
            :class:`py42.clients.users.UserClient`
        """
        return self._sdk_dependencies.user_client

    @property
    def devices(self):
        """A collection of methods for retrieving or updating data about devices in the Code42
        environment.

        Returns:
            :class:`py42.clients.devices.DeviceClient`
        """
        return self._sdk_dependencies.device_client

    @property
    def orgs(self):
        """A collection of methods for retrieving or updating data about organizations in the
        Code42 environment.

        Returns:
            :class:`py42.clients.orgs.OrgClient`
        """
        return self._sdk_dependencies.org_client

    @property
    def legalhold(self):
        """A collection of methods for retrieving and updating legal-hold matters, policies, and
        custodians.

        Returns:
            :class:`py42.clients.legalhold.LegalHoldClient`
        """
        return self._sdk_dependencies.legal_hold_client

    @property
    def usercontext(self):
        """A collection of methods related to getting information about the currently logged in
        user, such as the tenant ID.

        Returns:
            :class:`py42.usercontext.UserContext`
        """
        return self._sdk_dependencies.user_context

    @property
    def securitydata(self):
        """A collection of methods and properties for getting security data such as:
            * File events
            * Alerts
            * Security plan information

        Returns:
            :class:`py42.modules.securitydata.SecurityModule`
        """
        return self._sdk_dependencies.security_module

    @property
    def detectionlists(self):
        """A collection of properties each containing methods for managing specific detection
        lists, such as departing employees.

        Returns:
            :class:`py42.modules.detectionlists.DetectionListsModule`
        """
        return self._sdk_dependencies.detection_lists_module



class ArchiveModule(object):
    """A module for getting information about archives on storage nodes along with functionality
    for streaming a file from backup.
    """

    def __init__(self, archive_accessor_manager, archive_client):
        self._archive_accessor_manager = archive_accessor_manager
        self._archive_client = archive_client

    def stream_from_backup(
        self,
        file_path,
        device_guid,
        destination_guid=None,
        archive_password=None,
        encryption_key=None,
    ):
        """Streams a file from an archive to memory.
        `REST Documentation <https://console.us.code42.com/apidocviewer/#WebRestoreJobResult-get>`__

        Args:
            file_path (str): The path to the file in your archive.
            device_guid (str): The GUID for the device the file belongs to.
            destination_guid (str, optional): The GUID for the destination that stores the backup
                of the file. If None, it will use the first destination GUID it finds for your
                device. 'destination_guid' may be useful if the file is missing from one of your
                destinations or if you want to optimize performance. Defaults to None.
            archive_password (str, None): The password for archives that are protected with an
                additional password. This is only relevant to users with this level of archive
                security. Defaults to None.
            encryption_key (str, None): A custom encryption key for decryption an archive's file
                contents, necessary for restoring files. This is only relevant to users with
                this level of archive security. Defaults to None.
        Returns:
            :class:`py42.response.Py42Response`: A response containing the streamed content.

        Usage example::

            stream_response = sdk.archive.stream_from_backup("/full/path/to/file.txt", "1234567890")
            with open("/path/to/my/file", 'wb') as f:
                for chunk in stream_response.iter_content(chunk_size=128):
                    if chunk:
                        f.write(chunk)
        """
        archive_accessor = self._archive_accessor_manager.get_archive_accessor(
            device_guid,
            destination_guid=destination_guid,
            private_password=archive_password,
            encryption_key=encryption_key,
        )
        return archive_accessor.stream_from_backup(file_path)

    def get_backup_sets(self, device_guid, destination_guid):
        """Gets all backup set names/identifiers referring to single destination for a specific
        device.
        `Support Page <https://support.code42.com/Administrator/Cloud/Configuring/Backup_sets>`__

        Args:
            device_guid (str): The GUID for the device to get backup sets for.
            destination_guid (str): The GUID for the destination containing the archive to get
                backup sets for.

        Returns:
            :class:`py42.response.Py42Response`: A response containing the backup sets.
        """
        return self._archive_client.get_backup_sets(device_guid, destination_guid)

    def get_all_org_restore_history(self, days, org_id):
        """Gets all restore jobs from the past given days for the organization with the given ID.
        `REST Documentation <https://console.us.code42.com/apidocviewer/#RestoreHistory-get>`__

        Args:
            days (int): Number of days of restore history to retrieve.
            org_id (int): The ID for the organization to get restore history for.

        Returns:
            generator: An object that iterates over :class:`py42.response.Py42Response` objects
            that each contain a page of restore history.
        """
        return self._archive_client.get_all_restore_history(days, u"orgId", org_id)

    def get_all_user_restore_history(self, days, user_id):
        """Gets all restore jobs from the past given days for the user with the given ID.
        `REST Documentation <https://console.us.code42.com/apidocviewer/#RestoreHistory-get>`__

        Args:
            days (int): Number of days of restore history to retrieve.
            user_id (int): The ID for the user to get restore history for.

        Returns:
            generator: An object that iterates over :class:`py42.response.Py42Response` objects
            that each contain a page of restore history.
        """
        return self._archive_client.get_all_restore_history(days, u"userId", user_id)

    def get_all_device_restore_history(self, days, device_id):
        """Gets all restore jobs from the past given days for the device with the given ID.
        `REST Documentation <https://console.us.code42.com/apidocviewer/#RestoreHistory-get>`__

        Args:
            days (int): Number of days of restore history to retrieve.
            device_id (int): The ID for the device to get restore history for.

        Returns:
            generator: An object that iterates over :class:`py42.response.Py42Response` objects
            that each contain a page of restore history.
        """
        return self._archive_client.get_all_restore_history(days, u"computerId", device_id)



import json

from py42.clients import BaseClient
from py42.clients.util import get_all_pages


class OrgClient(BaseClient):
    """A client for interacting with Code42 organization APIs.

    Use the OrgClient to create and retrieve organizations. You can also use it to block and
    deactivate organizations.
    """

    def create_org(self, org_name, org_ext_ref=None, notes=None, parent_org_uid=None):
        """Creates a new organization.
        `REST Documentation <https://console.us.code42.com/apidocviewer/#Org-post>`__

        Args:
            org_name (str): The name of the new organization.
            org_ext_ref (str, optional): External reference information,
                such as a serial number, asset tag, employee ID, or help desk issue ID. Defaults to
                None.
            notes (str, optional): Descriptive information about the organization. Defaults to None.
            parent_org_uid (int, optional): The org UID for the parent organization. Defaults to
                None.

        Returns:
            :class:`py42.response.Py42Response`
        """
        uri = u"/api/Org/"
        data = {
            u"orgName": org_name,
            u"orgExtRef": org_ext_ref,
            u"notes": notes,
            u"parentOrgUid": parent_org_uid,
        }
        return self._session.post(uri, data=json.dumps(data))

    def get_by_id(self, org_id, **kwargs):
        """Gets the organization with the given ID.
        `REST Documentation <https://console.us.code42.com/apidocviewer/#Org-get>`__

        Args:
            org_id (int): An ID for an organization.

        Returns:
            :class:`py42.response.Py42Response`: A response containing the organization.
        """
        uri = u"/api/Org/{0}".format(org_id)
        return self._session.get(uri, params=kwargs)

    def get_by_uid(self, org_uid, **kwargs):
        """Gets the organization with the given UID.
        `REST Documentation <https://console.us.code42.com/apidocviewer/#Org-get>`__

        Args:
            org_uid (str): A UID for an organization.

        Returns:
            :class:`py42.response.Py42Response`: A response containing the organization.
        """
        uri = u"/api/Org/{0}".format(org_uid)
        params = dict(idType=u"orgUid", **kwargs)
        return self._session.get(uri, params=params)

    def get_by_name(self, org_name, **kwargs):
        """Gets the organization with the given name.
        `REST Documentation <https://console.us.code42.com/apidocviewer/#Org-get>`__

        Args:
            org_name (str): A name of an organization.

        Returns:
            list: A list of :class:`py42.response.Py42Response` objects each containing an
            organization that has the given name.
        """
        found_orgs = []
        response = self.get_all(**kwargs)
        for page in response:
            org_list = page[u"orgs"]
            for org in org_list:
                if org[u"orgName"] == org_name:
                    found_orgs.append(org)
        return found_orgs

    def _get_page(self, page_num=None, page_size=None, **kwargs):
        uri = u"/api/Org"
        params = dict(pgNum=page_num, pgSize=page_size, **kwargs)
        return self._session.get(uri, params=params)

    def get_all(self, **kwargs):
        """Gets all organizations.
        `REST Documentation <https://console.us.code42.com/apidocviewer/#Org-get>`__

        Returns:
            generator: An object that iterates over :class:`py42.response.Py42Response` objects
            that each contain a page of organizations.
        """
        return get_all_pages(self._get_page, u"orgs", **kwargs)

    def block(self, org_id):
        """Blocks the organization with the given org ID as well as its child organizations. A
        blocked organization will not allow any of its users or devices to log in. New
        registrations will be rejected and all currently logged in clients will be logged out.
        Backups continue for any devices that are still active.
        `Rest Documentation <https://console.us.code42.com/apidocviewer/#OrgBlock-put>`__

        Args:
            org_id (int): An ID for an organization.

        Returns:
            :class:`py42.response.Py42Response`
        """
        uri = u"/api/OrgBlock/{0}".format(org_id)
        return self._session.put(uri)

    def unblock(self, org_id):
        """Removes a block, if one exists, on an organization and its descendants with the given
        ID. All users in the organization remain blocked until they are unblocked individually.
        `REST Documentation <https://console.us.code42.com/apidocviewer/#OrgBlock-delete>`__

        Args:
            org_id (int): An ID for an organization.

        Returns:
            :class:`py42.response.Py42Response`
        """
        uri = u"/api/OrgBlock/{0}".format(org_id)
        return self._session.delete(uri)

    def deactivate(self, org_id):
        """Deactivates the organization with the given ID, including all users, plans, and
        devices. Backups stop and archives move to cold storage.
        `REST Documentation <https://console.us.code42.com/apidocviewer/#OrgDeactivation-put>`__

        Args:
            org_id (int): An ID for an organization.

        Returns:
            :class:`py42.response.Py42Response`
        """
        uri = u"/api/OrgDeactivation/{0}".format(org_id)
        return self._session.put(uri)

    def reactivate(self, org_id):
        """Reactivates the organization with the given ID. Backups are *not* restarted
        automatically.
        `REST Documentation <https://console.us.code42.com/apidocviewer/#OrgDeactivation-delete>`__

        Args:
            org_id (int): An ID for an organization.

        Returns:
            :class:`py42.response.Py42Response`
        """
        uri = u"/api/OrgDeactivation/{0}".format(org_id)
        return self._session.delete(uri)

    def get_current(self, **kwargs):
        """Gets the organization for the currently signed-in user.
        `REST Documentation <https://console.us.code42.com/apidocviewer/#Org-get>`__

        Returns:
            :class:`py42.response.Py42Response`: A response containing the organization for the
            currently signed-in user.
        """
        uri = u"/api/Org/my"
        return self._session.get(uri, params=kwargs)



import json

from requests import HTTPError

from py42._internal.clients import archive
from py42._internal.clients import key_value_store
from py42._internal.clients import securitydata
from py42.clients import administration, alerts, devices, legalhold, orgs, users
from py42.clients.detectionlists.departing_employee import DepartingEmployeeClient
from py42.clients.detectionlists.high_risk_employee import HighRiskEmployeeClient
from py42._internal.clients.detection_list_user import DetectionListUserClient
from py42.clients.file_event import FileEventClient
from py42.exceptions import Py42FeatureUnavailableError, Py42SessionInitializationError


class AuthorityClientFactory(object):
    def __init__(self, session):
        self.session = session

    def create_administration_client(self):
        return administration.AdministrationClient(self.session)

    def create_user_client(self):
        return users.UserClient(self.session)

    def create_device_client(self):
        return devices.DeviceClient(self.session)

    def create_org_client(self):
        return orgs.OrgClient(self.session)

    def create_legal_hold_client(self):
        return legalhold.LegalHoldClient(self.session)

    def create_archive_client(self):
        return archive.ArchiveClient(self.session)

    def create_security_client(self):
        return securitydata.SecurityClient(self.session)


class MicroserviceClientFactory(object):
    def __init__(
        self,
        authority_url,
        root_session,
        session_factory,
        user_context,
        key_value_store_client=None,
    ):
        self._authority_url = authority_url
        self._root_session = root_session
        self._session_factory = session_factory
        self._user_context = user_context
        self._key_value_store_client = key_value_store_client
        self._alerts_client = None
        self._departing_employee_client = None
        self._file_event_client = None
        self._high_risk_employee_client = None
        self._detection_list_user_client = None

    def get_alerts_client(self):
        if not self._alerts_client:
            session = self._get_jwt_session(u"AlertService-API_URL")
            self._alerts_client = alerts.AlertClient(session, self._user_context)
        return self._alerts_client

    def get_departing_employee_client(self):
        if not self._departing_employee_client:
            session = self._get_jwt_session(u"employeecasemanagement-API_URL")
            self._departing_employee_client = DepartingEmployeeClient(session, self._user_context)
        return self._departing_employee_client

    def get_file_event_client(self):
        if not self._file_event_client:
            session = self._get_jwt_session(u"FORENSIC_SEARCH-API_URL")
            self._file_event_client = FileEventClient(session)
        return self._file_event_client

    def _get_jwt_session(self, key):
        url = self._get_stored_value(key)
        return self._session_factory.create_jwt_session(url, self._root_session)

    def _get_stored_value(self, key):
        if not self._key_value_store_client:
            url = _hacky_get_microservice_url(self._root_session, u"simple-key-value-store")
            session = self._session_factory.create_anonymous_session(url)
            self._key_value_store_client = key_value_store.KeyValueStoreClient(session)
        return self._key_value_store_client.get_stored_value(key).text

    def get_high_risk_employee_client(self, user_client):
        if not self._high_risk_employee_client:
            session = self._get_jwt_session(u"employeecasemanagement-API_URL")
            self._high_risk_employee_client = HighRiskEmployeeClient(
                session, self._user_context, self.get_detection_list_user_client(), user_client
            )
        return self._high_risk_employee_client

    def get_detection_list_user_client(self):
        if not self._detection_list_user_client:
            session = self._get_jwt_session(u"employeecasemanagement-API_URL")
            self._detection_list_user_client = DetectionListUserClient(session, self._user_context)
        return self._detection_list_user_client


def _hacky_get_microservice_url(session, microservice_base_name):
    sts_url = _get_sts_base_url(session)
    return sts_url.replace(u"sts", microservice_base_name)


def _get_sts_base_url(session):
    uri = u"/api/ServerEnv"
    try:
        response = session.get(uri)
    except HTTPError as ex:
        raise Py42SessionInitializationError(ex)

    sts_base_url = None
    if response.text:
        response_json = json.loads(response.text)
        sts_base_url = response_json.get(u"stsBaseUrl")
    if not sts_base_url:
        raise Py42FeatureUnavailableError()
    return sts_base_url



import json

from py42.clients import BaseClient
from py42.clients.util import get_all_pages
from py42.exceptions import Py42Error


def _active_state_map(active):
    _map = {True: u"ACTIVE", False: u"INACTIVE", None: u"ALL"}
    try:
        return _map[active]
    except KeyError:
        raise Py42Error(
            "Invalid argument: '{0}'. active must be True, False, or None".format(active)
        )


class LegalHoldClient(BaseClient):
    """A client for interacting with Code42 Legal Hold APIs.

    The LegalHoldClient provides the ability to manage Code42 Legal Hold Policies and Matters.
    It can:
        - Create, view, and list all existing Policies.
        - Create, view, deactivate, reactivate, and list all existing Matters.
        - Add/remove Custodians from a Matter.
    """

    def create_policy(self, name, policy=None):
        """Creates a new Legal Hold Preservation Policy.
        `V4 REST Documentation <https://console.us.code42.com/swagger/#/legal-hold-policy/LegalHoldPolicy_Create>`__

        Args:
            name (str): The name of the new Policy.
            policy (dict, optional): The desired Preservation Policy settings as a dict. Defaults to
                None (where the server-default backup set is used).

        Returns:
            :class:`py42.response.Py42Response`
        """
        uri = u"/api/v4/legal-hold-policy/create"
        data = {u"name": name, u"policy": policy}
        return self._session.post(uri, data=json.dumps(data))

    def create_matter(self, name, hold_policy_uid, description=None, notes=None, hold_ext_ref=None):
        """Creates a new, active Legal Hold Matter.
        `REST Documentation <https://console.us.code42.com/apidocviewer/#LegalHold-post>`__

        Args:
            name (str): The name of the new Legal Hold Matter.
            hold_policy_uid (str): The identifier of the Preservation Policy that will apply to this
                Matter.
            description (str, optional): An optional description of the Matter. Defaults to None.
            notes (str, optional): Optional notes information. Defaults to None.
            hold_ext_ref (str, optional): Optional external reference information. Defaults to None.

        Returns:
            :class:`py42.response.Py42Response`
        """
        uri = u"/api/LegalHold"
        data = {
            u"name": name,
            u"holdPolicyUid": hold_policy_uid,
            u"description": description,
            u"notes": notes,
            u"holdExtRef": hold_ext_ref,
        }
        return self._session.post(uri, data=json.dumps(data))

    def get_policy_by_uid(self, legal_hold_policy_uid):
        """Gets a single Preservation Policy.
        `V4 REST Documentation <https://console.us.code42.com/swagger/#/legal-hold-policy/LegalHoldPolicy_View>`__

        Args:
            legal_hold_policy_uid (str): The identifier of the Preservation Policy.

        Returns:
            :class:`py42.response.Py42Response`: A response containing the Policy.
        """
        uri = u"/api/v4/legal-hold-policy/view"
        params = {u"legalHoldPolicyUid": legal_hold_policy_uid}
        return self._session.get(uri, params=params)

    def get_policy_list(self):
        """Gets a list of existing Preservation Policies.
        `V4 REST Documentation <https://console.us.code42.com/swagger/#/legal-hold-policy/LegalHoldPolicy_List>`__

        Returns:
            :class:`py42.response.Py42Response`: A response containing the list of Policies.
        """
        uri = u"/api/v4/legal-hold-policy/list"
        return self._session.get(uri)

    def get_matter_by_uid(self, legal_hold_uid):
        """Gets a single Legal Hold Matter.
        `REST Documentation <https://console.us.code42.com/apidocviewer/#LegalHold-get>`__

        Args:
            legal_hold_uid (str): The identifier of the Legal Hold Matter.

        Returns:
            :class:`py42.response.Py42Response`: A response containing the Matter.
        """
        uri = u"/api/LegalHold/{0}".format(legal_hold_uid)
        return self._session.get(uri)

    def _get_legal_holds_page(
        self,
        creator_user_uid=None,
        active=True,
        name=None,
        hold_ext_ref=None,
        page_num=None,
        page_size=None,
    ):
        active_state = _active_state_map(active)
        uri = u"/api/LegalHold"
        params = {
            u"creatorUserUid": creator_user_uid,
            u"activeState": active_state,
            u"name": name,
            u"holdExtRef": hold_ext_ref,
            u"pgNum": page_num,
            u"pgSize": page_size,
        }
        return self._session.get(uri, params=params)

    def get_all_matters(self, creator_user_uid=None, active=True, name=None, hold_ext_ref=None):
        """Gets all existing Legal Hold Matters.
        `REST Documentation <https://console.us.code42.com/apidocviewer/#LegalHold-get>`__

        Args:
            creator_user_uid (str, optional): Find Matters by the identifier of the user who created
                them. Defaults to None.
            active (bool or None, optional): Find Matters by their active state. True returns
                active Matters, False returns inactive Matters, None returns all Matters regardless
                of state. Defaults to True.
            name (str, optional): Find Matters with a 'name' that either equals or partially
                contains this value. Defaults to None.
            hold_ext_ref (str, optional): Find Matters having a matching external reference field.
                Defaults to None.

        Returns:
            generator: An object that iterates over :class:`py42.response.Py42Response` objects
            that each contain a page of Legal Hold Matters.
        """
        return get_all_pages(
            self._get_legal_holds_page,
            u"legalHolds",
            creator_user_uid=creator_user_uid,
            active=active,
            name=name,
            hold_ext_ref=hold_ext_ref,
        )

    def _get_legal_hold_memberships_page(
        self,
        legal_hold_membership_uid=None,
        legal_hold_uid=None,
        user_uid=None,
        user=None,
        active=True,
        page_num=None,
        page_size=None,
    ):
        active_state = _active_state_map(active)
        params = {
            u"legalHoldMembershipUid": legal_hold_membership_uid,
            u"legalHoldUid": legal_hold_uid,
            u"userUid": user_uid,
            u"user": user,
            u"activeState": active_state,
            u"pgNum": page_num,
            u"pgSize": page_size,
        }
        uri = u"/api/LegalHoldMembership"
        return self._session.get(uri, params=params)

    def get_all_matter_custodians(self, legal_hold_uid=None, user_uid=None, user=None, active=True):
        """Gets all Legal Hold memberships.

        Each user (Custodian) who has been added to a Legal Hold Matter is returned by the server as
        a LegalHoldMembership object in the response body.  If the object's active state is
        "INACTIVE", they have been removed from the Matter and are no longer subject to the Legal
        Hold retention rules. Users can be Custodians of multiple Legal Holds at once (and thus
        would be part of multiple LegalHoldMembership objects).
        `REST Documentation <https://console.us.code42.com/apidocviewer/#LegalHoldMembership-get>`__

        Args:
            legal_hold_uid (str, optional): Find LegalHoldMemberships for the Legal Hold Matter
                with this unique identifier. Defaults to None.
            user_uid (str, optional): Find LegalHoldMemberships for the user with this identifier.
                Defaults to None.
            user (str, optional): Find LegalHoldMemberships by flexibly searching on username,
                email, extUserRef, or last name. Will find partial matches. Defaults to None.
            active (bool or None, optional): Find LegalHoldMemberships by their active state. True
                returns active LegalHoldMemberships, False returns inactive LegalHoldMemberships,
                None returns all LegalHoldMemberships regardless of state. Defaults to True.

        Returns:
            generator: An object that iterates over :class:`py42.response.Py42Response` objects
            that each contain a page of LegalHoldMembership objects.
        """
        return get_all_pages(
            self._get_legal_hold_memberships_page,
            u"legalHoldMemberships",
            legal_hold_uid=legal_hold_uid,
            user_uid=user_uid,
            user=user,
            active=active,
        )

    def add_to_matter(self, user_uid, legal_hold_uid):
        """Add a user (Custodian) to a Legal Hold Matter.
        `REST Documentation <https://console.us.code42.com/apidocviewer/#LegalHoldMembership-post>`__

        Args:
            user_uid (str): The identifier of the user.
            legal_hold_uid (str): The identifier of the Legal Hold Matter.

        Returns:
            :class:`py42.response.Py42Response`
        """
        uri = u"/api/LegalHoldMembership"
        data = {u"legalHoldUid": legal_hold_uid, u"userUid": user_uid}
        return self._session.post(uri, data=json.dumps(data))

    def remove_from_matter(self, legal_hold_membership_uid):
        """Remove a user (Custodian) from a Legal Hold Matter.
        `REST Documentation <https://console.us.code42.com/apidocviewer/#LegalHoldMembershipDeactivation-post>`__

        Args:
            legal_hold_membership_uid (str): The identifier of the LegalHoldMembership representing
                the Custodian to Matter relationship.

        Returns:
            :class:`py42.response.Py42Response`
        """
        uri = u"/api/LegalHoldMembershipDeactivation"
        data = {u"legalHoldMembershipUid": legal_hold_membership_uid}
        return self._session.post(uri, data=json.dumps(data))

    def deactivate_matter(self, legal_hold_uid):
        """Deactivates and closes a Legal Hold Matter.
        `V4 REST Documentation <https://console.us.code42.com/swagger/#/legal-hold-deactivation/LegalHoldDeactivation_Update>`__

        Args:
            legal_hold_uid (str): The identifier of the Legal Hold Matter.

        Returns:
            :class:`py42.response.Py42Response`
        """
        uri = u"/api/v4/legal-hold-deactivation/update"
        data = {u"legalHoldUid": legal_hold_uid}
        return self._session.post(uri, data=json.dumps(data))

    def reactivate_matter(self, legal_hold_uid):
        """Reactivates and re-opens a closed Matter.
        `REST Documentation <https://console.us.code42.com/apidocviewer/#LegalHoldReactivation-put>`__

        Args:
            legal_hold_uid (str): The identifier of the Legal Hold Matter.

        Returns:
            :class:`py42.response.Py42Response`
        """
        uri = u"/api/LegalHoldReactivation/{0}".format(legal_hold_uid)
        return self._session.put(uri)



from py42._internal.compat import str
from py42.sdk.queries import BaseQuery


class AlertQuery(BaseQuery):
    """Helper class for building Code42 Alert queries.

    An AlertQuery instance's ``all()`` and ``any()`` take one or more :class:`FilterGroup`
    objects to construct a query that can be passed to the :meth:`AlertClient.search()` method.
    ``all()`` returns results that match all of the provided filter criteria, ``any()`` will return
    results that match any of the filters.

    For convenience, the :class:`AlertQuery` constructor does the same as ``all()``.

    A tenant ID is required in either the constructor or ``all()`` or ``any()``. You can get the
    tenant ID from the method :meth:`SDKClient.usercontext.get_current_tenant_id()`.

    Usage example:::
        state_filter = AlertState.eq(AlertState.OPEN)
        rule_name_filter = RuleName.contains("EmailRule")
        tenant_id = sdk.usercontext.get_current_tenant_id()
        query = AlertQuery(tenant_id).all(state_filter, rule_name_filter)
    """

    def __init__(self, tenant_id, *args, **kwargs):
        super(AlertQuery, self).__init__(*args, **kwargs)
        self._tenant_id = tenant_id
        self.sort_key = u"CreatedAt"
        self.page_number = 0
        self.sort_direction = u"desc"

    def __str__(self):
        groups_string = u",".join(str(group_item) for group_item in self._filter_group_list)
        json = u'{{"tenantId":"{0}", "groupClause":"{1}", "groups":[{2}], "pgNum":{3}, "pgSize":{4}, "srtDirection":"{5}", "srtKey":"{6}"}}'.format(
            self._tenant_id,
            self._group_clause,
            groups_string,
            self.page_number,
            self.page_size,
            self.sort_direction,
            self.sort_key,
        )
        return json



class BaseQuery(object):
    def __init__(self, *args, **kwargs):
        self._filter_group_list = list(args)
        self._group_clause = kwargs.get(u"group_clause", u"AND")
        self.page_size = 10000
        self.sort_direction = u"asc"

        # Override
        self.page_number = None
        self.sort_key = None

    @classmethod
    def any(cls, *args):
        return cls(*args, group_clause=u"OR")

    @classmethod
    def all(cls, *args):
        return cls(*args)



from py42._internal.compat import str
from py42.sdk.queries import BaseQuery
from py42.sdk.queries.query_filter import (
    QueryFilterStringField,
    create_filter_group,
    create_query_filter,
)


class FileEventQuery(BaseQuery):
    """Helper class for building Code42 Forensic Search queries.

    A FileEventQuery instance's ``all()`` and ``any()`` take one or more :class:`FilterGroup`
    objects to construct a query that can be passed to the :meth:`FileEventClient.search()` method.
    ``all()`` returns results that match all of the provided filter criteria, ``any()`` will return
    results that match any of the filters.

    For convenience, the :class:`FileEventQuery` constructor does the same as ``all()``.

    Usage example::

        email_filter = EmailSender.is_in(["test.user@example.com", "test.sender@example.com"])
        exposure_filter = ExposureType.exists()

        query = FileEventQuery.all(email_filter, exposure_filter)

    """

    def __init__(self, *args, **kwargs):
        super(FileEventQuery, self).__init__(*args, **kwargs)
        self.sort_key = u"eventId"
        self.page_number = 1

    def __str__(self):
        groups_string = u",".join(str(group_item) for group_item in self._filter_group_list)
        json = u'{{"groupClause":"{0}", "groups":[{1}], "pgNum":{2}, "pgSize":{3}, "srtDir":"{4}", "srtKey":"{5}"}}'.format(
            self._group_clause,
            groups_string,
            self.page_number,
            self.page_size,
            self.sort_direction,
            self.sort_key,
        )
        return json


def create_exists_filter_group(term):
    filter_list = [create_query_filter(term, u"EXISTS")]
    return create_filter_group(filter_list, u"AND")


def create_not_exists_filter_group(term):
    filter_list = [create_query_filter(term, u"DOES_NOT_EXIST")]
    return create_filter_group(filter_list, u"AND")


def create_greater_than_filter_group(term, value):
    filter_list = [create_query_filter(term, u"GREATER_THAN", value)]
    return create_filter_group(filter_list, u"AND")


def create_less_than_filter_group(term, value):
    filter_list = [create_query_filter(term, u"LESS_THAN", value)]
    return create_filter_group(filter_list, u"AND")


class FileEventFilterStringField(QueryFilterStringField):
    """Helper class for creating filters with the ``EXISTS``/``NOT_EXISTS`` filter clauses."""

    @classmethod
    def exists(cls):
        """Returns a :class:`FilterGroup` to find events where filter data exists."""
        return create_exists_filter_group(cls._term)

    @classmethod
    def not_exists(cls):
        """Returns a :class:`FilterGroup` to find events where filter data does not exist."""
        return create_not_exists_filter_group(cls._term)


class FileEventFilterComparableField(object):
    """Helper class for creating filters with the ``GREATER_THAN``/``LESS_THAN`` filter clauses."""

    _term = u"override_boolean_field_name"

    @classmethod
    def greater_than(cls, value):
        """Returns a :class:`FilterGroup` to find events where filter data is greater than the
        provided value.
        """
        value = int(value)
        return create_greater_than_filter_group(cls._term, value)

    @classmethod
    def less_than(cls, value):
        """Returns a :class:`FilterGroup` to find events where filter data is less than than the
        provided value.
        """
        value = int(value)
        return create_less_than_filter_group(cls._term, value)



import base64
import json

from py42._internal.auth_handling import TokenProvider

V3_AUTH = u"v3_user_token"


class BasicAuthProvider(TokenProvider):
    def __init__(self, username, password):
        super(BasicAuthProvider, self).__init__()
        cred_bytes = base64.b64encode(u"{0}:{1}".format(username, password).encode(u"utf-8"))
        self._base64_credentials = cred_bytes.decode(u"utf-8")

    def get_secret_value(self, force_refresh=False):
        return self._base64_credentials


class C42ApiV3TokenProvider(TokenProvider):
    def __init__(self, auth_session):
        super(C42ApiV3TokenProvider, self).__init__()
        self._auth_session = auth_session

    def get_secret_value(self, force_refresh=False):
        uri = u"/c42api/v3/auth/jwt"
        params = {u"useBody": True}
        response = self._auth_session.get(uri, params=params)
        return response[V3_AUTH]


class C42ApiV1TokenProvider(TokenProvider):
    def __init__(self, auth_session):
        super(C42ApiV1TokenProvider, self).__init__()
        self._auth_session = auth_session

    def get_secret_value(self, force_refresh=False):
        uri = u"/api/AuthToken"
        response = self._auth_session.post(uri, data=None)
        return u"{0}-{1}".format(response[0], response[1])


class C42APITmpAuthProvider(TokenProvider):
    def __init__(self):
        super(C42APITmpAuthProvider, self).__init__()
        self._cached_info = None

    def get_login_info(self):
        if self._cached_info is None:
            response = self.get_tmp_auth_token()  # pylint: disable=assignment-from-no-return
            self._cached_info = response
        return self._cached_info

    def get_tmp_auth_token(self):
        pass

    def get_secret_value(self, force_refresh=False):
        if force_refresh or self._cached_info is None:
            self.get_login_info()
        return self._cached_info[u"loginToken"]


class C42APILoginTokenProvider(C42APITmpAuthProvider):
    def __init__(self, auth_session, user_id, device_guid, destination_guid):
        super(C42APILoginTokenProvider, self).__init__()
        self._auth_session = auth_session
        self._user_id = user_id
        self._device_guid = device_guid
        self._destination_guid = destination_guid

    def get_tmp_auth_token(self):
        uri = u"/api/LoginToken"
        data = {
            u"userId": self._user_id,
            u"sourceGuid": self._device_guid,
            u"destinationGuid": self._destination_guid,
        }
        response = self._auth_session.post(uri, data=json.dumps(data))
        return response


class C42APIStorageAuthTokenProvider(C42APITmpAuthProvider):
    def __init__(self, auth_session, plan_uid, destination_guid):
        super(C42APIStorageAuthTokenProvider, self).__init__()
        self._auth_session = auth_session
        self._plan_uid = plan_uid
        self._destination_guid = destination_guid

    def get_tmp_auth_token(self):
        uri = u"/api/StorageAuthToken"
        data = {u"planUid": self._plan_uid, u"destinationGuid": self._destination_guid}
        response = self._auth_session.post(uri, data=json.dumps(data))
        return response


class StorageTokenProviderFactory(object):
    def __init__(self, auth_session, security_client, device_client):
        self._auth_session = auth_session
        self._security_client = security_client
        self._device_client = device_client

    def create_security_archive_locator(self, plan_uid, destination_guid):
        return C42APIStorageAuthTokenProvider(self._auth_session, plan_uid, destination_guid)

    def create_backup_archive_locator(self, device_guid, destination_guid=None):
        if destination_guid is None:
            response = self._device_client.get_by_guid(device_guid, include_backup_usage=True)
            if destination_guid is None:
                # take the first destination guid we find
                destination_list = response["backupUsage"]
                if not destination_list:
                    raise Exception(
                        u"No destinations found for device guid: {0}".format(device_guid)
                    )
                destination_guid = destination_list[0][u"targetComputerGuid"]

        return C42APILoginTokenProvider(self._auth_session, u"my", device_guid, destination_guid)



from __future__ import print_function

import json as json_lib
from threading import Lock

import requests.adapters

from py42._internal.compat import str, urljoin, urlparse
import py42.settings as settings
import py42.settings.debug as debug
from py42.exceptions import raise_py42_error
from py42.response import Py42Response


class Py42Session(object):
    def __init__(self, session, host_address, auth_handler=None):
        self._initialized = False
        self._needs_auth_renewal_check = False
        self._auth_lock = Lock()
        self._session = session
        adapter = requests.adapters.HTTPAdapter(pool_connections=20, pool_maxsize=20)
        if not host_address.startswith(u"http://") and not host_address.startswith(u"https://"):
            host_address = u"https://{0}".format(host_address)

        self._host_address = host_address
        self._auth_handler = auth_handler

        self._session.proxies = settings.proxies
        self._session.verify = settings.verify_ssl_certs
        self._session.mount(u"https://", adapter)
        self._session.mount(u"http://", adapter)

        self._host_address = host_address
        parsed_host = urlparse(self._host_address)
        host = parsed_host.netloc

        self._session.headers = {
            u"Accept": u"application/json",
            u"Content-Type": u"application/json",
            u"Host": host,
            u"User-Agent": settings.get_user_agent_string(),
            u"Accept-Encoding": u"gzip, deflate",
            u"Connection": u"keep-alive",
        }

    @property
    def host_address(self):
        return self._host_address

    @property
    def headers(self):
        return self._session.headers

    @property
    def cookies(self):
        return self._session.cookies

    @property
    def proxies(self):
        return self._session.proxies

    def get(self, url, **kwargs):
        return self.request(u"GET", url, **kwargs)

    def options(self, url, **kwargs):
        return self.request(u"OPTIONS", url, **kwargs)

    def head(self, url, **kwargs):
        return self.request(u"HEAD", url, **kwargs)

    def post(self, url, data=None, json=None, **kwargs):
        return self.request(u"POST", url, data=data, json=json, **kwargs)

    def put(self, url, data=None, **kwargs):
        return self.request(u"PUT", url, data=data, **kwargs)

    def patch(self, url, data=None, **kwargs):
        return self.request(u"PATCH", url, data=data, **kwargs)

    def delete(self, url, **kwargs):
        return self.request(u"DELETE", url, **kwargs)

    def request(self, method, url, **kwargs):
        try:
            url = urljoin(self._host_address, url)
            json = kwargs.get(u"json")

            if json is not None:
                kwargs[u"data"] = json_lib.dumps(_filter_out_none(json))
            if u"json" in kwargs:
                del kwargs[u"json"]

            self._renew_authentication(use_cache=True)

            tries = 0
            max_tries = 2
            while tries < max_tries:
                response, unauthorized = self._try_make_request(method, url, **kwargs)
                tries += 1

                if unauthorized and tries < max_tries:
                    self._renew_authentication()
                    continue

                if response.status_code >= 400:
                    response.raise_for_status()

                if not kwargs.get(u"stream"):
                    response.encoding = u"utf-8"  # setting this manually speeds up read times

                return Py42Response(response)
        except requests.HTTPError as err:
            raise_py42_error(err)

    def _try_make_request(
        self,
        method,
        url,
        params=None,
        data=None,
        headers=None,
        cookies=None,
        files=None,
        auth=None,
        timeout=60,
        allow_redirects=True,
        proxies=None,
        hooks=None,
        stream=None,
        verify=None,
        cert=None,
    ):

        self._print_request(method, url, params=params, data=data)

        response = self._session.request(
            method,
            url,
            params=params,
            data=data,
            headers=headers,
            cookies=cookies,
            files=files,
            auth=auth,
            timeout=timeout,
            allow_redirects=allow_redirects,
            proxies=proxies,
            hooks=hooks,
            stream=stream,
            verify=verify,
            cert=cert,
        )

        unauthorized = self._auth_handler and self._auth_handler.response_indicates_unauthorized(
            response
        )

        return response, unauthorized

    def _renew_authentication(self, use_cache=False):
        if self._auth_handler:
            # if multiple threads try to authenticate at once, only the first one actually does.
            # the rest will just wait for that authentication to complete.
            self._needs_auth_renewal_check = True
            with self._auth_lock:
                # only get new credentials if this is the first time or we want fresh ones
                should_renew = (
                    not self._initialized or not use_cache
                ) and self._needs_auth_renewal_check
                if should_renew:
                    self._auth_handler.renew_authentication(self, use_cache=use_cache)
                    self._needs_auth_renewal_check = False

        # if there's no auth handler or we handled auth without errors, initialization is done.
        self._initialized = True

    def _print_request(self, method, url, params=None, data=None):
        if debug.will_print_for(debug.INFO):
            print(u"{0}{1}".format(str(method).ljust(8), url))
        if debug.will_print_for(debug.TRACE):
            if self.headers:
                _print_dict(self.headers, u"  headers")
        if debug.will_print_for(debug.DEBUG):
            if params:
                _print_dict(params, u"  params")
            if data:
                _print_dict(data, u"  data")


def _filter_out_none(_dict):
    return {key: _dict[key] for key in _dict if _dict[key] is not None}


def _print_dict(dict_, label=None):
    if label:
        print(label, end=" ")
    print(json_lib.dumps(dict_, indent=4))



from threading import Lock

from requests import HTTPError

from py42._internal.compat import str
from py42.exceptions import Py42StorageSessionInitializationError


class StorageSessionManager(object):
    def __init__(self, session_factory, session_cache=None):
        self._session_factory = session_factory
        self._session_cache = session_cache or {}
        self._list_update_lock = Lock()

    def get_saved_session_for_url(self, url):
        return self._session_cache.get(url.lower())

    def get_storage_session(self, token_provider):
        try:
            url = token_provider.get_login_info()[u"serverUrl"]
            session = self.get_saved_session_for_url(url)
            if session is None:
                with self._list_update_lock:
                    session = self.get_saved_session_for_url(url)
                    if session is None:
                        session = self.create_storage_session(url, token_provider)
                        self._session_cache.update({url.lower(): session})
        except HTTPError as ex:
            message = u"Failed to create or retrieve session, caused by: {0}".format(str(ex))
            raise Py42StorageSessionInitializationError(message)
        return session

    def create_storage_session(self, url, token_provider):
        return self._session_factory.create_storage_session(url, token_provider)



from py42.clients import BaseClient


class KeyValueStoreClient(BaseClient):
    def __init__(self, session):
        super(KeyValueStoreClient, self).__init__(session)

    def get_stored_value(self, key):
        uri = u"/v1/{0}".format(key)
        return self._session.get(uri)



from py42._internal.archive_access import ArchiveAccessorManager
from py42._internal.client_factories import AuthorityClientFactory, MicroserviceClientFactory
from py42._internal.clients.storage import StorageClientFactory
from py42._internal.storage_session_manager import StorageSessionManager
from py42._internal.token_providers import StorageTokenProviderFactory
from py42.modules import archive as archive_module, detectionlists, securitydata as sec_module
from py42.usercontext import UserContext


def _get_storage_client_factory(session_factory, archive_locator_factory):
    storage_session_manager = StorageSessionManager(session_factory)
    return StorageClientFactory(storage_session_manager, archive_locator_factory)


class SDKDependencies(object):
    def __init__(self, host_address, session_factory, root_session):
        self._set_v3_session(host_address, session_factory, root_session)

        # authority clients
        authority_client_factory = AuthorityClientFactory(self.session)
        self.administration_client = authority_client_factory.create_administration_client()
        self.user_client = authority_client_factory.create_user_client()
        self.device_client = authority_client_factory.create_device_client()
        self.org_client = authority_client_factory.create_org_client()
        self.legal_hold_client = authority_client_factory.create_legal_hold_client()
        self.archive_client = authority_client_factory.create_archive_client()
        self.security_client = authority_client_factory.create_security_client()
        self.user_context = UserContext(self.administration_client)

        archive_locator_factory = StorageTokenProviderFactory(
            self.session, self.security_client, self.device_client
        )

        self.storage_client_factory = _get_storage_client_factory(
            session_factory, archive_locator_factory
        )

        archive_accessor_manager = ArchiveAccessorManager(
            self.archive_client, self.storage_client_factory
        )

        microservice_client_factory = MicroserviceClientFactory(
            host_address, root_session, session_factory, self.user_context
        )

        # modules (feature sets that combine info from multiple clients)
        self.archive_module = archive_module.ArchiveModule(
            archive_accessor_manager, self.archive_client
        )
        self.security_module = sec_module.SecurityModule(
            self.security_client, self.storage_client_factory, microservice_client_factory
        )
        self.detection_lists_module = detectionlists.DetectionListsModule(
            microservice_client_factory, self.user_client
        )

    def _set_v3_session(self, host_address, session_factory, root_session):
        self.root_session = root_session
        self.session = session_factory.create_jwt_session(host_address, root_session)
        self._test_session(self.session, u"/api/User/my")

    @staticmethod
    def _test_session(session, test_uri):
        response = session.get(test_uri)
        return 200 <= response.status_code < 300



from py42.sdk.queries.query_filter import QueryFilterStringField


class EmailPolicyName(QueryFilterStringField):
    """Class that filters events based on the email DLP policy that detected this file (applies to
    emails sent via Microsoft Office 365 only).
    """

    _term = u"emailDlpPolicyNames"


class EmailSubject(QueryFilterStringField):
    """Class that filters events based on the email's subject (applies to email events only)."""

    _term = u"emailSubject"


class EmailRecipients(QueryFilterStringField):
    """Class that filters events based on the email's recipient list (applies to email events only)."""

    _term = u"emailRecipients"


class EmailSender(QueryFilterStringField):
    """Class that filters events based on the email's sender (applies to email events only)."""

    _term = u"emailSender"


class EmailFrom(QueryFilterStringField):
    """Class that filters events based on the display name of the email's sender, as it appears in
    the \"From:\" field in the email (applies to email events only).
    """

    _term = u"emailFrom"



from py42._internal.clients.storage.archive import StorageArchiveClient
from py42._internal.clients.storage.securitydata import StorageSecurityClient
from py42.clients import BaseClient


class StorageClient(BaseClient):
    def __init__(self, session):
        super(StorageClient, self).__init__(session)
        self._archive_client = StorageArchiveClient(session)
        self._security_client = StorageSecurityClient(session)

    @property
    def archive(self):
        return self._archive_client

    @property
    def securitydata(self):
        return self._security_client


class StorageClientFactory(object):
    def __init__(self, storage_session_manager, token_provider_factory):
        self._storage_session_manager = storage_session_manager
        self._token_provider_factory = token_provider_factory

    def from_device_guid(self, device_guid, destination_guid=None):
        token_provider = self._token_provider_factory.create_backup_archive_locator(
            device_guid, destination_guid
        )
        session = self._storage_session_manager.get_storage_session(token_provider)
        return StorageClient(session)

    def from_plan_info(self, plan_uid, destination_guid):
        token_provider = self._token_provider_factory.create_security_archive_locator(
            plan_uid, destination_guid
        )
        session = self._storage_session_manager.get_storage_session(token_provider)
        return StorageClient(session)



from py42._internal.compat import str


class Py42Error(Exception):
    """A generic, Py42 custom base exception."""


class Py42ArchiveFileNotFoundError(Py42Error):
    """An exception raised when a resource file is not found or the path is invalid."""

    def __init__(self, device_guid, file_path):
        message = u"File not found in archive " u"for device {0} at path {1}".format(
            device_guid, file_path
        )
        super(Py42ArchiveFileNotFoundError, self).__init__(message)


class Py42FeatureUnavailableError(Py42Error):
    """An exception raised when a requested feature is not supported in your Code42 instance."""

    def __init__(self):
        super(Py42FeatureUnavailableError, self).__init__(
            u"You may be trying to use a feature that is unavailable in your environment."
        )


class Py42SessionInitializationError(Py42Error):
    """An exception raised when a user session is invalid. A session might be invalid due to
    session timeout, invalid token, etc.
    """

    def __init__(self, exception):
        error_message = (
            u"An error occurred while requesting "
            u"server environment information, caused by {0}".format(str(exception))
        )
        super(Py42SessionInitializationError, self).__init__(error_message)


class Py42SecurityPlanConnectionError(Py42Error):
    """An exception raised when the user is not authorized to access the requested resource."""

    def __init__(self, error_message):
        super(Py42SecurityPlanConnectionError, self).__init__(error_message)


class Py42StorageSessionInitializationError(Py42Error):
    """An exception raised when the user is not authorized to initialize a storage session. This
    may occur when trying to restore a file or trying to get legacy security events."""

    def __init__(self, error_message):
        super(Py42StorageSessionInitializationError, self).__init__(error_message)


class Py42HTTPError(Py42Error):
    """A base custom class to manage all HTTP errors raised by an API endpoint."""

    def __init__(self, http_response):
        message = u"Failure in HTTP call {0}".format(str(http_response))
        super(Py42HTTPError, self).__init__(message)
        self._response = http_response

    @property
    def response(self):
        """The response containing the HTTP error."""
        return self._response


class Py42BadRequestError(Py42HTTPError):
    """A wrapper to represent an HTTP 400 error."""

    def __init__(self, exception):
        super(Py42BadRequestError, self).__init__(str(exception))


class Py42UnauthorizedError(Py42HTTPError):
    """A wrapper to represent an HTTP 401 error."""

    def __init__(self, exception):
        super(Py42UnauthorizedError, self).__init__(str(exception))


class Py42ForbiddenError(Py42HTTPError):
    """A wrapper to represent an HTTP 403 error."""

    def __init__(self, exception):
        super(Py42ForbiddenError, self).__init__(str(exception))


class Py42NotFoundError(Py42HTTPError):
    """A wrapper to represent an HTTP 404 error."""

    def __init__(self, exception):
        super(Py42NotFoundError, self).__init__(str(exception))


class Py42InternalServerError(Py42HTTPError):
    """A wrapper to represent an HTTP 500 error."""

    def __init__(self, exception):
        super(Py42InternalServerError, self).__init__(str(exception))


def raise_py42_error(raised_error):
    """Raises the appropriate :class:`py42.exceptions.Py42HttpError` based on the given
    error's response status code.
    """
    if raised_error.response.status_code == 400:
        raise Py42BadRequestError(raised_error)
    elif raised_error.response.status_code == 401:
        raise Py42UnauthorizedError(raised_error)
    elif raised_error.response.status_code == 403:
        raise Py42ForbiddenError(raised_error)
    elif raised_error.response.status_code == 404:
        raise Py42NotFoundError(raised_error)
    elif 500 <= raised_error.response.status_code < 600:
        raise Py42InternalServerError(raised_error)
    else:
        raise Py42HTTPError(raised_error.response)



import sys

from py42.__version__ import __version__

proxies = None

# Controls whether we verify the server's certificate.
# True, False, or a path to a CA bundle to use.
verify_ssl_certs = True

items_per_page = 1000

_custom_user_suffix = u""
_python_version = u"{0}.{1}.{2}".format(
    sys.version_info[0], sys.version_info[1], sys.version_info[2]
)


def get_user_agent_string():
    return u"py42 {0} python {1}{2}".format(__version__, _python_version, _custom_user_suffix)


def set_user_agent_suffix(suffix):
    global _custom_user_suffix
    _custom_user_suffix = u" {0}".format(suffix)



from py42.sdk.queries.query_filter import (
    QueryFilterStringField,
    create_filter_group,
    create_query_filter,
    QueryFilterTimestampField,
)


def create_contains_filter_group(term, value):
    filter_list = [create_query_filter(term, u"CONTAINS", value)]
    return create_filter_group(filter_list, u"AND")


def create_not_contains_filter_group(term, value):
    filter_list = [create_query_filter(term, u"DOES_NOT_CONTAIN", value)]
    return create_filter_group(filter_list, u"AND")


class AlertQueryFilterStringField(QueryFilterStringField):
    @classmethod
    def contains(cls, value):
        return create_contains_filter_group(cls._term, value)

    @classmethod
    def not_contains(cls, value):
        return create_not_contains_filter_group(cls._term, value)


class DateObserved(QueryFilterTimestampField):
    """Class that filters alerts based on the timestamp the alert was triggered."""

    _term = u"CreatedAt"


class Actor(AlertQueryFilterStringField):
    """Class that filters alerts based on the username that originated the event(s) that
    triggered the alert."""

    _term = u"actor"


class RuleName(AlertQueryFilterStringField):
    """Class that filters alerts based on rule name."""

    _term = u"name"


class Description(AlertQueryFilterStringField):
    """Class that filters alerts based on rule description text."""

    _term = u"description"


class Severity(QueryFilterStringField):
    """Class that filters alerts based on severity.

    Available options are:
        - :attr:`Severity.HIGH`
        - :attr:`Severity.MEDIUM`
        - :attr:`Severity.LOW`
    """

    _term = u"severity"

    HIGH = u"HIGH"
    MEDIUM = u"MEDIUM"
    LOW = u"LOW"


class AlertState(QueryFilterStringField):
    """Class that filters alerts based on alert state.

    Available options are:
        - :attr:`AlertState.OPEN`
        - :attr:`AlertState.DISMISSED`
    """

    _term = u"state"

    OPEN = u"OPEN"
    DISMISSED = u"RESOLVED"



import json

from py42.clients import BaseClient
from py42.clients.util import get_all_pages


class ArchiveClient(BaseClient):
    def get_data_key_token(self, device_guid):
        uri = u"/api/DataKeyToken"
        data = {u"computerGuid": device_guid}
        return self._session.post(uri, data=json.dumps(data))

    def get_backup_sets(self, device_guid, destination_guid):
        uri = u"/c42api/v3/BackupSets/{}/{}".format(device_guid, destination_guid)
        return self._session.get(uri)

    def get_all_restore_history(self, days, id_type, id_value, **kwargs):
        return get_all_pages(
            self._get_restore_history_page,
            u"restoreEvents",
            days=days,
            id_type=id_type,
            id_value=id_value,
            **kwargs
        )

    def _get_restore_history_page(self, days, id_type, id_value, page_num, page_size, **kwargs):
        uri = u"/api/RestoreHistory"
        params = dict(days=days, pgNum=page_num, pgSize=page_size, **kwargs)
        params[id_type] = id_value
        return self._session.get(uri, params=params)

    def get_web_restore_info(self, src_guid, dest_guid):
        uri = u"/api/WebRestoreInfo"
        params = {u"srcGuid": src_guid, u"destGuid": dest_guid}
        return self._session.get(uri, params=params)



class DetectionListsModule(object):
    def __init__(self, microservice_client_factory, user_client):
        self._microservice_client_factory = microservice_client_factory
        self._user_client = user_client
        self._detection_list_user_client = None

    @property
    def departing_employee(self):
        return self._microservice_client_factory.get_departing_employee_client()

    @property
    def high_risk_employee(self):
        return self._microservice_client_factory.get_high_risk_employee_client(self._user_client)

    def create(self, username):
        """Create a detection list profile for a user.

        Args:
            username (str): Username of the user.

        Returns:
            :class:`py42.response.Py42Response`
        """
        self._detection_list_user_client = (
            self._microservice_client_factory.get_detection_list_user_client()
        )
        return self._detection_list_user_client.create(username)

    def get(self, username):
        """Get user details by username.

        Args:
            username (str): Username of the user.

        Returns:
            :class:`py42.response.Py42Response`
        """
        self._detection_list_user_client = (
            self._microservice_client_factory.get_detection_list_user_client()
        )
        return self._detection_list_user_client.get(username)

    def get_by_id(self, user_id):
        """Get user details by user_id.

        Args:
            user_id (str or int): Id of the user.

        Returns:
            :class:`py42.response.Py42Response`
        """
        self._detection_list_user_client = (
            self._microservice_client_factory.get_detection_list_user_client()
        )
        return self._detection_list_user_client.get_by_id(user_id)

    def update_notes(self, user_id, notes):
        """Add or update notes related to the user.

        Args:
            user_id (str or int): The user_id whose notes need to be updated.
            notes (str): User profile notes.

        Returns:
            :class:`py42.response.Py42Response`
        """
        self._detection_list_user_client = (
            self._microservice_client_factory.get_detection_list_user_client()
        )
        self._detection_list_user_client.update_notes(user_id, notes)

    def add_risk_tag(self, user_id, tags):
        """Add one or more tags.

        Args:
            user_id (str or int): The user_id whose tag(s) needs to be updated.
            tags (str or list of str ): A single tag or multiple tags in a list to be added.
               e.g u"tag1" or ["tag1", "tag2"], for python version 2.X, pass u"str" instead of "str"

        Returns:
            :class:`py42.response.Py42Response`
        """
        self._detection_list_user_client = (
            self._microservice_client_factory.get_detection_list_user_client()
        )
        self._detection_list_user_client.add_risk_tags(user_id, tags)

    def remove_risk_tag(self, user_id, tags):
        """Remove one or more tags.

        Args:
            user_id (str or int): The user_id whose tag(s) needs to be removed.
            tags (str or list of str ): A single tag or multiple tags in a list to be removed.
               e.g u"tag1" or ["tag1", "tag2"], for python version 2.X, pass u"str" instead of "str"

        Returns:
            :class:`py42.response.Py42Response`
        """
        self._detection_list_user_client = (
            self._microservice_client_factory.get_detection_list_user_client()
        )
        self._detection_list_user_client.remove_risk_tags(user_id, tags)

    def add_cloud_alias(self, user_id, aliases):
        """Add one or more cloud alias.

        Args:
            user_id (str or int): The user_id whose alias(es) need to be updated.
            aliases (str or list of str ): A single alias or multiple aliases in a list to be added.
                e.g u"x" or ["email@id", "y"], for python version 2.X, pass u"str" instead of "str"

        Returns:
            :class:`py42.response.Py42Response`
        """
        self._detection_list_user_client = (
            self._microservice_client_factory.get_detection_list_user_client()
        )
        self._detection_list_user_client.add_cloud_aliases(user_id, aliases)

    def remove_cloud_alias(self, user_id, aliases):
        """Remove one or more cloud alias.

        Args:
            user_id (str or int): The user_id whose alias(es) need to be removed.
            aliases (str or list of str ): A single alias or multiple aliases in a list to be removed.
                e.g u"x" or ["email@id", "y"], for python version 2.X, pass u"str" instead of "str"

        Returns:
            :class:`py42.response.Py42Response`
        """
        self._detection_list_user_client = (
            self._microservice_client_factory.get_detection_list_user_client()
        )
        self._detection_list_user_client.remove_cloud_aliases(user_id, aliases)



import json

from py42._internal.compat import str
from py42.clients import BaseClient


class AlertClient(BaseClient):
    """A client for interacting with Code42 security alerts.

    The AlertClient has the ability to search, resolve, and reopen alerts.
    Also, it can get the details for the file event query for the event that triggered the alert.
    """

    _uri_prefix = u"/svc/api/v1/{0}"

    def __init__(self, session, user_context):
        super(AlertClient, self).__init__(session)
        self._user_context = user_context

    def search(self, query):
        """Searches alerts using the given :class:`py42.sdk.queries.alerts.alert_query.AlertQuery`.

        Args:
            query (:class:`py42.sdk.queries.alerts.alert_query.AlertQuery`): An alert query.
                See `userguides.searches` to learn more about how to construct a query.

        Returns:
            :class:`py42.response.Py42Response`: A response containing the alerts that match the given
            query.
        """
        query = str(query)
        uri = self._uri_prefix.format(u"query-alerts")
        return self._session.post(uri, data=query)

    def get_details(self, alert_ids, tenant_id=None):
        """Gets the details for the alerts with the given IDs, including the file event query that,
        when passed into a search, would result in events that could have triggered the alerts.

        Args:
            alert_ids (iter[str]): The IDs of the alerts for which you want to get details for.
            tenant_id (str, optional): The tenant ID for the tenant that the alerts belong to.
                When given None, it uses the currently logged in user's tenant ID. Defaults to
                None.

        Returns:
            :class:`py42.response.Py42Response`: A response containing the alert details.
        """
        if type(alert_ids) is not list:
            alert_ids = [alert_ids]
        tenant_id = tenant_id if tenant_id else self._user_context.get_current_tenant_id()
        uri = self._uri_prefix.format(u"query-details")
        data = {u"tenantId": tenant_id, u"alertIds": alert_ids}
        return self._session.post(uri, data=json.dumps(data))

    def resolve(self, alert_ids, tenant_id=None, reason=None):
        """Resolves the alerts with the given IDs.

        Args:
            alert_ids (iter[str]): The IDs for the alerts to resolve.
            tenant_id (str, optional): The ID for the tenant that the alerts belong to.
                When given None, it uses the currently logged in user's tenant ID. Defaults to
                None.
            reason (str, optional): The reason the alerts are now resolved. Defaults to None.

        Returns:
            :class:`py42.response.Py42Response`
        """
        if type(alert_ids) is not list:
            alert_ids = [alert_ids]
        tenant_id = tenant_id if tenant_id else self._user_context.get_current_tenant_id()
        reason = reason if reason else u""
        uri = self._uri_prefix.format(u"resolve-alert")
        data = {u"tenantId": tenant_id, u"alertIds": alert_ids, u"reason": reason}
        return self._session.post(uri, data=json.dumps(data))

    def reopen(self, alert_ids, tenant_id=None, reason=None):
        """Reopens the resolved alerts with the given IDs.

        Args:
            alert_ids (iter[str]): The IDs for the alerts to reopen.
            tenant_id (str, optional): The ID for the tenant that the alerts belong to. When given
                None, it uses the currently logged in user's tenant ID. Defaults to None.
            reason (str, optional): The reason the alerts are reopened. Defaults to None.

        Returns:
            :class:`py42.response.Py42Response`
        """
        if type(alert_ids) is not list:
            alert_ids = [alert_ids]
        tenant_id = tenant_id if tenant_id else self._user_context.get_current_tenant_id()
        uri = self._uri_prefix.format(u"reopen-alert")
        data = {u"tenantId": tenant_id, u"alertIds": alert_ids, u"reason": reason}
        return self._session.post(uri, data=json.dumps(data))



import json

from py42._internal.compat import str
from py42.clients import BaseClient


class DetectionListUserClient(BaseClient):
    """Administrator utility to manage High Risk employees information.

    `Support Documentation <https://support.code42.com/Administrator/Cloud/Monitoring_and_managing/Detection_list_management_APIs>`__
    """

    _api_version = u"v2"
    _uri_prefix = u"/svc/api/{0}".format(_api_version)
    _resource = u"/user"

    def __init__(self, session, user_context):
        super(DetectionListUserClient, self).__init__(session)
        self._user_context = user_context

    def _make_uri(self, action):
        return u"{0}{1}{2}".format(self._uri_prefix, self._resource, action)

    def create(self, username):
        """Create a detection list profile for a user.

        Args:
            username (str): Username of the user.

        Returns:
            :class:`py42.response.Py42Response`
        """
        data = {
            u"tenantId": self._user_context.get_current_tenant_id(),
            u"userName": username,
            u"notes": "",
            u"riskFactors": [],
            u"cloudUsernames": [],
        }
        uri = self._make_uri(u"/create")
        return self._session.post(uri, data=json.dumps(data))

    def get_by_id(self, user_id):
        """Get user details by user id.

        Args:
            user_id (str or int): Id of the user.

        Returns:
            :class:`py42.response.Py42Response`
        """
        data = {u"tenantId": self._user_context.get_current_tenant_id(), u"userId": user_id}
        uri = self._make_uri(u"/getbyid")
        return self._session.post(uri, data=json.dumps(data))

    def get(self, username):
        """Get user details by username.

        Args:
            username (str): Username of the user.

        Returns:
            :class:`py42.response.Py42Response`
        """
        data = {u"tenantId": self._user_context.get_current_tenant_id(), u"username": username}
        uri = self._make_uri(u"/getbyusername")
        return self._session.post(uri, data=json.dumps(data))

    def update_notes(self, user_id, notes):
        """Add or update notes related to the user.

        Args:
            user_id (str or int): The user_id whose notes need to be updated.
            notes (str): User profile notes.

        Returns:
            :class:`py42.response.Py42Response`
        """
        data = {
            u"tenantId": self._user_context.get_current_tenant_id(),
            u"userId": user_id,
            u"notes": notes,
        }
        uri = self._make_uri(u"/updatenotes")
        return self._session.post(uri, data=json.dumps(data))

    def add_risk_tags(self, user_id, tags):
        """Add one or more tags.

        Args:
            user_id (str or int): The user_id whose tag(s) needs to be updated.
            tags (str or list of str ): A single tag or multiple tags in a list to be added.
               e.g u"tag1" or ["tag1", "tag2"], for python version 2.X, pass u"str" instead of "str"

        Returns:
            :class:`py42.response.Py42Response`
        """

        if type(tags) is str:
            tags = [tags]

        data = {
            u"tenantId": self._user_context.get_current_tenant_id(),
            u"userId": user_id,
            u"riskFactors": tags,
        }
        uri = self._make_uri(u"/addriskfactors")
        return self._session.post(uri, data=json.dumps(data))

    def remove_risk_tags(self, user_id, tags):
        """Remove one or more tags.Args:

        Args:
            user_id (str or int): The user_id whose tag(s) needs to be removed.
            tags (str or list of str ): A single tag or multiple tags in a list to be removed.
               e.g u"tag1" or ["tag1", "tag2"], for python version 2.X, pass u"str" instead of "str"

        Returns:
            :class:`py42.response.Py42Response`
        """
        if type(tags) is str:
            tags = [tags]

        data = {
            u"tenantId": self._user_context.get_current_tenant_id(),
            u"userId": user_id,
            u"riskFactors": tags,
        }
        uri = self._make_uri(u"/removeriskfactors")
        return self._session.post(uri, data=json.dumps(data))

    def add_cloud_aliases(self, user_id, aliases):
        """Add one or more cloud alias.

        Args:
            user_id (str or int): The user_id whose alias(es) need to be updated.
            aliases (str or list of str ): A single alias or multiple aliases in a list to be added.
                e.g u"x" or ["email@id", "y"], for python version 2.X, pass u"str" instead of "str"

        Returns:
            :class:`py42.response.Py42Response`
        """
        if type(aliases) is str:
            aliases = [aliases]

        data = {
            u"tenantId": self._user_context.get_current_tenant_id(),
            u"userId": user_id,
            u"cloudUsernames": aliases,
        }
        uri = self._make_uri(u"/addcloudusernames")
        return self._session.post(uri, data=json.dumps(data))

    def remove_cloud_aliases(self, user_id, aliases):
        """Remove one or more cloud alias.

        Args:
            user_id (str or int): The user_id whose alias(es) need to be removed.
            aliases (str or list of str ): A single alias or multiple aliases in a list to be removed.
                e.g u"x" or ["email@id", "y"], for python version 2.X, pass u"str" instead of "str"

        Returns:
            :class:`py42.response.Py42Response`
        """
        if type(aliases) is str:
            aliases = [aliases]

        data = {
            u"tenantId": self._user_context.get_current_tenant_id(),
            u"userId": user_id,
            u"cloudUsernames": aliases,
        }
        uri = self._make_uri(u"/removecloudusernames")
        return self._session.post(uri, data=json.dumps(data))



import json

from py42.clients import BaseClient
from py42.clients.util import get_all_pages
from py42.util import convert_timestamp_to_str


class DepartingEmployeeClient(BaseClient):
    """A client for interacting with Code42 Departing Employee APIs."""

    _uri_prefix = u"/svc/api/v1/departingemployee/{0}"

    def __init__(self, session, user_context):
        super(DepartingEmployeeClient, self).__init__(session)
        self._user_context = user_context

    def create(
        self, username, tenant_id=None, notes=None, departure_epoch=None, cloud_usernames=None
    ):
        """Adds a user to Departing Employees.
        `REST Documentation <https://ecm-default.prod.ffs.us2.code42.com/svc/swagger/index.html#/DepartingEmployeeCase/DepartingEmployeeCase_Create>`__

        Args:
            username (str): The username of the departing employee.
            tenant_id (str, optional): The identifier of the Customer tenant the user is in.
                Defaults to None (the tenant_id of SDK authorization user will be used).
            notes (str, optional): Optional descriptive information. Defaults to None.
            departure_epoch (int, optional): Employee departure date as POSIX timestamp. Defaults
                to None.
            cloud_usernames (list, optional): List of alternate usernames for this user from
                external data sources being searched by Forensic File Search (Google Drive, Box,
                OneDrive, Gmail, Office 365). Defaults to None.

        Returns:
            :class:`py42.response.Py42Response`
        """

        tenant_id = tenant_id if tenant_id else self._user_context.get_current_tenant_id()
        cloud_usernames = cloud_usernames if cloud_usernames else []
        departure_date = (
            convert_timestamp_to_str(departure_epoch) if departure_epoch else departure_epoch
        )
        data = {
            u"userName": username,
            u"tenantId": tenant_id,
            u"notes": notes,
            u"departureDate": departure_date,
            u"alertsEnabled": True,
            u"cloudUsernames": cloud_usernames,
        }
        uri = self._uri_prefix.format(u"create")
        return self._session.post(uri, data=json.dumps(data))

    def resolve(self, case_id, tenant_id=None):
        """Removes a user from Departing Employees.
        `REST Documentation <https://ecm-default.prod.ffs.us2.code42.com/svc/swagger/index.html#/DepartingEmployeeCase/DepartingEmployeeCase_ResolveCase>`__

        Args:
            case_id (str): The identifier of the Departing Employee.
            tenant_id (str, optional): The identifier of the Customer tenant the case is in.
                Defaults to None (the tenant_id of SDK authorization user will be used).

        Returns:
            :class:`py42.response.Py42Response`
        """

        tenant_id = tenant_id if tenant_id else self._user_context.get_current_tenant_id()
        uri = self._uri_prefix.format(u"resolve")
        data = {u"caseId": case_id, u"tenantId": tenant_id}
        return self._session.post(uri, data=json.dumps(data))

    def _get_departing_employees_page(
        self,
        tenant_id=None,
        departing_on_or_after_epoch=None,
        sort_key=u"CREATED_AT",
        sort_direction=u"DESC",
        page_num=None,
        page_size=None,
    ):

        tenant_id = tenant_id if tenant_id else self._user_context.get_current_tenant_id()
        departing_on_or_after_date = (
            convert_timestamp_to_str(departing_on_or_after_epoch)
            if departing_on_or_after_epoch
            else departing_on_or_after_epoch
        )
        uri = self._uri_prefix.format(u"search")
        data = {
            u"tenantId": tenant_id,
            u"pgSize": 100,
            u"pgNum": page_num,
            u"departingOnOrAfter": departing_on_or_after_date,
            u"srtKey": sort_key,
            u"srtDirection": sort_direction,
        }
        return self._session.post(uri, data=json.dumps(data))

    def get_all(
        self,
        tenant_id=None,
        departing_on_or_after_epoch=None,
        sort_key=u"CREATED_AT",
        sort_direction=u"DESC",
    ):
        """Gets all Departing Employees.

        Args:
            tenant_id (str, optional): The identifier of the Customer tenant. Defaults to None (the
                tenant_id of SDK authorization user will be used).
            departing_on_or_after_epoch (int, optional): Filter results by the departure date of
                employee, requires a POSIX timestamp. Defaults to None.
            sort_key (str, optional): Key to sort results on. Options: (``CREATED_AT``,
                ``DEPARTURE_DATE``, ``DISPLAY_NAME``, ``NUM_EVENTS``, ``TOTAL_BYTES``). Defaults to
                ``CREATED_AT``.
            sort_direction (str, optional): Sort direction. Options: (``ASC``, ``DESC``). Defaults
                to ``DESC``.

        Returns:
            generator: An object that iterates over :class:`py42.response.Py42Response` objects
            that each contain a page of departing employees.
        """
        return get_all_pages(
            self._get_departing_employees_page,
            u"cases",
            tenant_id=tenant_id,
            departing_on_or_after_epoch=departing_on_or_after_epoch,
            sort_key=sort_key,
            sort_direction=sort_direction,
        )

    def toggle_alerts(self, tenant_id=None, alerts_enabled=True):
        """Enable or disable email alerting on Departing Employee exposure events.
        `REST Documentation <https://ecm-default.prod.ffs.us2.code42.com/svc/swagger/index.html#/DepartingEmployeeCase/DepartingEmployeeCase_ToggleAlerts>`__

        Args:
            tenant_id (str, optional): The identifier of the Customer tenant. Defaults to None (the
                tenant_id of SDK authorization user will be used).
            alerts_enabled (bool): Set alerting to on (True) or off (False). Defaults to True.

        Returns:
            :class:`py42.response.Py42Response`
        """
        tenant_id = tenant_id if tenant_id else self._user_context.get_current_tenant_id()
        uri = self._uri_prefix.format(u"togglealerts")
        data = {u"tenantId": tenant_id, u"alertsEnabled": alerts_enabled}
        return self._session.post(uri, data=json.dumps(data))

    def get_by_username(self, username, tenant_id=None):
        """Gets Departing Employee case detail for a given user.

        Arguments:
            username (str): Username of the Departing Employee to retrieve case info on.
            tenant_id (str, optional): The identifier of the Customer tenant the case is in.
                Defaults to None (the tenant_id of SDK authorization user will be used).

        Returns:
            :class:`py42.response.Py42Response`: A response containing the Departing Employee case.
        """
        tenant_id = tenant_id if tenant_id else self._user_context.get_current_tenant_id()
        case_id = self._get_case_id_from_username(tenant_id, username)
        return self.get_by_id(case_id, tenant_id)

    def get_by_id(self, case_id, tenant_id=None):
        """Gets details about a Departing Employee case.
        `REST Documentation <https://ecm-default.prod.ffs.us2.code42.com/svc/swagger/index.html#/DepartingEmployeeCase/DepartingEmployeeCase_Details>`__

        Args:
            case_id (str): Identifier of the Departing Employee case.
            tenant_id (str, optional): The identifier of the Customer tenant the case is in.
                Defaults to None (the tenant_id of SDK authorization user will be used).

        Returns:
            :class:`py42.response.Py42Response`: A response containing the Departing Employee case.
        """
        tenant_id = tenant_id if tenant_id else self._user_context.get_current_tenant_id()
        uri = self._uri_prefix.format(u"details")
        data = {u"tenantId": tenant_id, u"caseId": case_id}
        return self._session.post(uri, data=json.dumps(data))

    def update(
        self,
        case_id,
        tenant_id=None,
        display_name=None,
        notes=None,
        departure_epoch=None,
        cloud_usernames=None,
    ):
        """Add or modify details of an existing Departing Employee case.
        `REST Documentation <https://ecm-default.prod.ffs.us2.code42.com/svc/swagger/index.html#/DepartingEmployeeCase/DepartingEmployeeCase_Update>`__

        Args:
            case_id (str): Identifier of the Departing Employee case.
            tenant_id (str, optional): The identifier of the Customer tenant the case is in.
                Defaults to None (the tenant_id of SDK authorization user will be used).
            display_name (str, optional): The display name for the Departing Employee case. This
                defaults to username when adding a new Departing Employee, so it can be used to make
                the UI more user-friendly if your Organization has usernames that don't correspond
                to real names. Defaults to None.
            notes (str, optional): Optional descriptive information. Defaults to None.
            departure_epoch (int, optional): Employee departure date as POSIX timestamp. Defaults
                to None.
            cloud_usernames (list, optional): List of alternate usernames for this user from
                external data sources being searched by Forensic File Search (Google Drive, Box,
                OneDrive, Gmail, Office 365). Defaults to None.

        Returns:
                :class:`py42.response.Py42Response`
        """

        tenant_id = tenant_id if tenant_id else self._user_context.get_current_tenant_id()

        # The behavior of the `update` API is to clear values that are not provided.
        # Therefore, we get current values first as to prevent clearing them when not provided.
        case = json.loads(self._get_case_by_id(case_id).text)

        display_name = display_name if display_name else case.get(u"displayName")
        notes = notes if notes else case.get(u"notes")
        departure_date = (
            convert_timestamp_to_str(departure_epoch)
            if departure_epoch
            else case.get(u"departureDate")
        )
        alerts_enabled = case.get(u"alertsEnabled")
        status = case.get(u"status")
        cloud_usernames = cloud_usernames if cloud_usernames else case.get(u"cloudUsernames")

        uri = self._uri_prefix.format(u"update")
        data = {
            u"tenantId": tenant_id,
            u"caseId": case_id,
            u"displayName": display_name,
            u"notes": notes,
            u"departureDate": departure_date,
            u"alertsEnabled": alerts_enabled,
            u"status": status,
            u"cloudUsernames": cloud_usernames,
        }
        return self._session.post(uri, data=json.dumps(data))

    def _get_case_id_from_username(self, tenant_id, username):
        case = self._get_case_from_username(tenant_id, username)
        if case is not None:
            return case.get(u"caseId")

    def _get_case_from_username(self, tenant_id, username):
        matches = None
        for page in self._get_all_departing_employees(tenant_id):
            matches = [c for c in page if c[u"userName"] == username]
            if matches:
                break
        return matches[0] if matches else None

    def _get_all_departing_employees(self, tenant_id):
        for page in self.get_all(tenant_id):
            yield page[u"cases"]

    def _get_case_by_id(self, case_id):
        return self.get_by_id(case_id)



# -*- coding: utf-8 -*-

from datetime import datetime

from py42._internal.compat import str
from py42.util import convert_datetime_to_timestamp_str, convert_timestamp_to_str


def create_query_filter(term, operator, value=None):
    return QueryFilter(term, operator, value)


def create_eq_filter_group(term, value):
    filter_list = [create_query_filter(term, u"IS", value)]
    return create_filter_group(filter_list, u"AND")


def create_is_in_filter_group(term, value_list):
    filter_list = [create_query_filter(term, u"IS", value) for value in value_list]
    return create_filter_group(filter_list, u"OR" if len(filter_list) > 1 else u"AND")


def create_not_in_filter_group(term, value_list):
    filter_list = [create_query_filter(term, u"IS_NOT", value) for value in value_list]
    return create_filter_group(filter_list, u"AND")


def create_not_eq_filter_group(term, value):
    filter_list = [create_query_filter(term, u"IS_NOT", value)]
    return create_filter_group(filter_list, u"AND")


def create_on_or_after_filter_group(term, value):
    filter_list = [create_query_filter(term, u"ON_OR_AFTER", value)]
    return create_filter_group(filter_list, u"AND")


def create_on_or_before_filter_group(term, value):
    filter_list = [create_query_filter(term, u"ON_OR_BEFORE", value)]
    return create_filter_group(filter_list, u"AND")


def create_in_range_filter_group(term, start_value, end_value):
    filter_list = [
        create_query_filter(term, u"ON_OR_AFTER", start_value),
        create_query_filter(term, u"ON_OR_BEFORE", end_value),
    ]
    return create_filter_group(filter_list, u"AND")


def create_filter_group(query_filter_list, filter_clause):
    return FilterGroup(query_filter_list, filter_clause)


class QueryFilterStringField(object):
    """Helper class for creating filters where the search value is a string."""

    _term = u"override_string_field_name"

    @classmethod
    def eq(cls, value):
        """Returns a :class:`FilterGroup` to find events where the filter equals provided ``value``.

        Args:
            value (str): The value to match file events on.
        """
        return create_eq_filter_group(cls._term, value)

    @classmethod
    def not_eq(cls, value):
        """Returns a :class:`FilterGroup` to find events where the filter is not equal to provided ``value``.

        Args:
            value (str): The value to exclude file events on.
        """
        return create_not_eq_filter_group(cls._term, value)

    @classmethod
    def is_in(cls, value_list):
        """Returns a :class:`FilterGroup` to find events where the filter is in provided ``value_list``.

        Args:
            value_list (list): The list of values to match file events on.
        """
        return create_is_in_filter_group(cls._term, value_list)

    @classmethod
    def not_in(cls, value_list):
        """Returns a :class:`FilterGroup` to find events where the filter is not in provided ``value_list``.

        Args:
            value_list (list): The list of values to exclude file events on.
        """
        return create_not_in_filter_group(cls._term, value_list)


class QueryFilterTimestampField(object):
    """Helper class for creating filters where the search value is a timestamp."""

    _term = u"override_timestamp_field_name"

    @classmethod
    def on_or_after(cls, value):
        """Returns a :class:`FilterGroup` to find events where the filter timestamp is on or after provided
        `value`.
        """
        formatted_timestamp = convert_timestamp_to_str(value)
        return create_on_or_after_filter_group(cls._term, formatted_timestamp)

    @classmethod
    def on_or_before(cls, value):
        """Returns a :class:`FilterGroup` to find events where the filter timestamp is on or before provided
        `value`.
        """
        formatted_timestamp = convert_timestamp_to_str(value)
        return create_on_or_before_filter_group(cls._term, formatted_timestamp)

    @classmethod
    def in_range(cls, start_value, end_value):
        """Returns a :class:`FilterGroup` to find events where the filter timestamp is in range between provided
        `start_value` and `end_value`.
        """
        formatted_start_time = convert_timestamp_to_str(start_value)
        formatted_end_time = convert_timestamp_to_str(end_value)
        return create_in_range_filter_group(cls._term, formatted_start_time, formatted_end_time)

    @classmethod
    def on_same_day(cls, value):
        """Returns a :class:`FilterGroup` to find events where the filter timestamp is within the same calendar
        day as provided `value`.
        """
        date_from_value = datetime.utcfromtimestamp(value)
        start_time = datetime(
            date_from_value.year, date_from_value.month, date_from_value.day, 0, 0, 0
        )
        end_time = datetime(
            date_from_value.year, date_from_value.month, date_from_value.day, 23, 59, 59
        )
        formatted_start_time = convert_datetime_to_timestamp_str(start_time)
        formatted_end_time = convert_datetime_to_timestamp_str(end_time)
        return create_in_range_filter_group(cls._term, formatted_start_time, formatted_end_time)


class QueryFilterBooleanField(object):
    """Helper class for creating filters where the search value is a boolean."""

    _term = u"override_boolean_field_name"

    @classmethod
    def is_true(cls):
        """Returns a :class:`FilterGroup` to find events where the filter state is True."""
        return create_eq_filter_group(cls._term, u"TRUE")

    @classmethod
    def is_false(cls):
        """Returns a :class:`FilterGroup` to find events where the filter state is False."""
        return create_eq_filter_group(cls._term, u"FALSE")


class QueryFilter(object):
    """Class for constructing a single filter object for use in a Forensic Search query.

    When :func:`str()` is called on a :class:`QueryFilter` instance, the (``term``, ``operator``,
    ``value``) attribute combination is transformed into a JSON string to be used as part of a
    Forensic Search or Alert query.
    """

    _term = None

    def __init__(self, term, operator, value=None):
        self._term = term
        self._operator = operator
        self._value = value

    def __str__(self):
        value = u"null" if self._value is None else u'"{0}"'.format(self._value)
        return u'{{"operator":"{0}", "term":"{1}", "value":{2}}}'.format(
            self._operator, self._term, value
        )


class FilterGroup(object):
    """Class for constructing a logical sub-group of related filters from a list of QueryFilter
    objects. Takes a list of QueryFilter objects and combines them logically using the passed in
    filter clause (``AND`` or ``OR``).

    When :func:`str()` is called on a :class:`FilterGroup` instance, the combined filter items are
    transformed into a JSON string to be used as part of a Forensic Search or Alert query.
    """

    def __init__(self, filter_list, filter_clause=u"AND"):
        self._filter_list = filter_list
        self._filter_clause = filter_clause

    def __str__(self):
        filters_string = u",".join(str(filter_item) for filter_item in self._filter_list)
        return u'{{"filterClause":"{0}", "filters":[{1}]}}'.format(
            self._filter_clause, filters_string
        )



from py42._internal.compat import str
from py42.clients import BaseClient


class FileEventClient(BaseClient):
    """A client for querying Code42 Forensic Search events."""

    def search(self, query):
        """Searches for file events matching query criteria.
        `REST Documentation <https://forensicsearch-east.us.code42.com/forensic-search/queryservice/swagger-ui.html#/file-event-controller/searchEventsUsingPOST>`__

        Args:
            query (:class:`FileEventQuery` or str): A composed FileEventQuery object or the raw
                query as a JSON formatted string.

        Returns:
            :class:`py42.response.Py42Response`: A response containing the query results.
        """
        query = str(query)
        uri = u"/forensic-search/queryservice/api/v1/fileevent"
        return self._session.post(uri, data=query)



import json

from py42.clients import BaseClient
from py42.clients.util import get_all_pages


class DeviceClient(BaseClient):
    """A class to interact with Code42 device/computer APIs."""

    def _get_page(
        self,
        active=None,
        blocked=None,
        org_uid=None,
        user_uid=None,
        destination_guid=None,
        include_backup_usage=None,
        include_counts=True,
        page_num=None,
        page_size=None,
        q=None,
    ):
        uri = u"/api/Computer"
        params = {
            u"active": active,
            u"blocked": blocked,
            u"orgUid": org_uid,
            u"userUid": user_uid,
            u"targetComputerGuid": destination_guid,
            u"incBackupUsage": include_backup_usage,
            u"incCounts": include_counts,
            u"pgNum": page_num,
            u"pgSize": page_size,
            u"q": q,
        }

        return self._session.get(uri, params=params)

    def get_all(
        self,
        active=None,
        blocked=None,
        org_uid=None,
        user_uid=None,
        destination_guid=None,
        include_backup_usage=None,
        include_counts=True,
        q=None,
        **kwargs
    ):
        """Gets all device information.

        When no arguments are passed, all records are returned. To filter results, specify
        respective arguments. For example, to retrieve all active and blocked devices, pass
        active=true and blocked=true.

        `REST Documentation <https://console.us.code42.com/apidocviewer/#Computer-get>`__

        Args:
            active (bool, optional): Filters results by device state. When set to True, gets all
                active devices. When set to False, gets all deactivated devices. When set to None
                or excluded, gets all devices regardless of state. Defaults to None.
            blocked (bool, optional): Filters results by blocked status, True or False. Defaults
                to None.
            org_uid (int, optional): An ID of an Organization. Defaults to None.
            user_uid (int, optional): An ID of a User. Defaults to None.
            destination_guid (str of int, optional): The GUID of the server that the desktop
                clients back up their files to. Defaults to None.
            include_backup_usage (type, bool): A flag to denote whether to include destination and
                its backup summary. Defaults to None.
            include_counts (bool, optional): A flag to denote whether to include total, warning,
                and critical counts. Defaults to True.
            q (type, optional): Searches results flexibly by incomplete GUID, host-name,
                computer name, etc. Defaults to None.

        Returns:
            generator: An object that iterates over :class:`py42.response.Py42Response` objects
            that each contain a page of devices.

            * For a logged in ordinary end user, it returns all the user's devices.

            * For an organization administrator, it returns all the devices in the organization.

            * For a cross-organization administrator, it returns all the devices from all the user's organizations.

            * Finally, for are a customer cloud administrator, it returns all devices in all organizations.
        """

        return get_all_pages(
            self._get_page,
            u"computers",
            active=active,
            blocked=blocked,
            org_uid=org_uid,
            user_uid=user_uid,
            destination_guid=destination_guid,
            include_backup_usage=include_backup_usage,
            include_counts=include_counts,
            q=q,
            **kwargs
        )

    def get_by_id(self, device_id, include_backup_usage=None, **kwargs):
        """Gets device information by ID.

        `REST Documentation <https://console.us.code42.com/apidocviewer/#Computer-get>`__

        Args:
            device_id (int): The ID of the device.
            include_backup_usage (bool, optional): A flag to denote whether to include its
              backup summary. Defaults to None.

        Returns:
            :class:`py42.response.Py42Response`: A response containing device information.
        """
        uri = u"/api/Computer/{0}".format(device_id)
        params = dict(incBackupUsage=include_backup_usage, **kwargs)
        return self._session.get(uri, params=params)

    def get_by_guid(self, guid, include_backup_usage=None, **kwargs):
        """Gets device information by GUID.

        `REST Documentation <https://console.us.code42.com/apidocviewer/#Computer-get>`__

        Args:
            guid (str): The GUID of the device.
            include_backup_usage (bool, optional): A flag to denote whether to include its
              backup summary. Defaults to None.

        Returns:
            :class:`py42.response.Py42Response`: A response containing device information.
        """
        uri = u"/api/Computer/{0}".format(guid)
        params = dict(idType=u"guid", incBackupUsage=include_backup_usage, **kwargs)
        return self._session.get(uri, params=params)

    def block(self, device_id):
        """Blocks a device causing the user not to be able to login or restore.

        `REST Documentation <https://console.us.code42.com/apidocviewer/#ComputerBlock>`__

        Args:
            device_id (int): The ID of the device.

        Returns:
            :class:`py42.response.Py42Response`
        """
        uri = u"/api/ComputerBlock/{0}".format(device_id)
        return self._session.put(uri)

    def unblock(self, device_id):
        """Unblocks a device, permitting a user to be able to login and restore again.

        `REST Documentation <https://console.us.code42.com/apidocviewer/#ComputerBlock>`__

        Args:
            device_id (int): The ID of the device.

        Returns:
            :class:`py42.response.Py42Response`
        """
        uri = u"/api/ComputerBlock/{0}".format(device_id)
        return self._session.delete(uri)

    def deactivate(self, device_id):
        """Deactivates a device, causing backups to stop and archives to go to cold storage.

        `REST Documentation <https://console.us.code42.com/apidocviewer/#ComputerDeactivation>`__

        Args:
            device_id (int): The ID of the device.

        Returns:
            :class:`py42.response.Py42Response`
        """
        uri = u"/api/v4/computer-deactivation/update"
        data = {u"id": device_id}
        return self._session.post(uri, data=json.dumps(data))

    def reactivate(self, device_id):
        """Activates a previously deactivated device.

        `REST Documentation <https://console.us.code42.com/apidocviewer/#ComputerDeactivation>`__

        Args:
            device_id (int): The ID of the device.

        Returns:
            :class:`py42.response.Py42Response`
        """
        uri = u"/api/v4/computer-deactivation/remove"
        data = {u"id": device_id}
        return self._session.post(uri, data=json.dumps(data))

    def deauthorize(self, device_id):
        """Deauthorizes the device with the given ID. If used on a cloud connector device, it will
            remove the authorization token for that account.

        `REST Documentation <https://console.us.code42.com/apidocviewer/#ComputerDeauthorization>`__

        Args:
            device_id (int): The ID of the device.

        Returns:
            :class:`py42.response.Py42Response`
        """
        uri = u"/api/ComputerDeauthorization/{0}".format(device_id)
        return self._session.put(uri)

    def get_settings(self, guid, keys=None):
        """Gets settings of the device.

        `REST Documentation <https://console.us.code42.com/apidocviewer/#DeviceSetting>`__

        Args:
            guid (str): The GUID of the device.
            keys (type, optional): A comma separated list of device keys. Defaults to None.

        Returns:
            :class:`py42.response.Py42Response`: A response containing settings information.
        """
        uri = u"/api/v4/device-setting/view"
        params = {u"guid": guid, u"keys": keys}
        return self._session.get(uri, params=params)



class TokenProvider(object):
    def get_secret_value(self, force_refresh=False):
        pass


class AuthHandler(object):
    def __init__(self, token_provider, session_modifier):
        self._token_provider = token_provider
        self._session_modifier = session_modifier

    def renew_authentication(self, session, use_cache=False):
        secret = self._token_provider.get_secret_value(force_refresh=not use_cache)
        self._session_modifier.modify_session(session, secret)

    @staticmethod
    def response_indicates_unauthorized(response):
        return response.status_code == 401


class HeaderModifier(object):
    def __init__(self, header_name=u"Authorization", value_format=u"{0}"):
        self._header_name = header_name
        self._value_format = value_format

    def modify_session(self, session, value):
        session.headers.update({self._header_name: self._value_format.format(value)})



from py42.sdk.queries.fileevents.file_event_query import FileEventFilterStringField
from py42.sdk.queries.query_filter import QueryFilterTimestampField


class EventTimestamp(QueryFilterTimestampField):
    """Class that filters events based on the timestamp of the event that occurred."""

    _term = u"eventTimestamp"


class EventType(FileEventFilterStringField):
    """Class that filters file events based on event type.

    Available event types are provided as class attributes:

        - :attr:`EventType.CREATED`
        - :attr:`EventType.DELETED`
        - :attr:`EventType.EMAILED`
        - :attr:`EventType.MODIFIED`
        - :attr:`EventType.READ_BY_APP`

    Example::

        filter = EventType.isin([EventType.READ_BY_APP, EventType.EMAILED])

    """

    _term = u"eventType"

    CREATED = u"CREATED"
    MODIFIED = u"MODIFIED"
    DELETED = u"DELETED"
    READ_BY_APP = u"READ_BY_APP"
    EMAILED = u"EMAILED"


class InsertionTimestamp(QueryFilterTimestampField):
    """Class that filters events based on the timestamp of when the event was actually added to the
    event store (which can be after the event occurred on the device itself).

    `value` must be a POSIX timestamp. (see the :ref:`Dates <anchor_dates>` section of the Basics
    userguide for details on timestamp arguments in py42)
    """

    _term = u"insertionTimestamp"


class Source(FileEventFilterStringField):
    _term = u"source"



import json
from threading import Lock

from requests.exceptions import HTTPError

from py42.exceptions import Py42SecurityPlanConnectionError, raise_py42_error


class SecurityModule(object):
    def __init__(self, security_client, storage_client_factory, microservices_client_factory):
        self._security_client = security_client
        self._storage_client_factory = storage_client_factory
        self._microservices_client_factory = microservices_client_factory
        self._client_cache = {}
        self._client_cache_lock = Lock()

    @property
    def alerts(self):
        """A collection of methods for retrieving and updating data about security alerts."""
        return self._microservices_client_factory.get_alerts_client()

    def get_security_plan_storage_info_list(self, user_uid):
        """Gets IDs (plan UID, node GUID, and destination GUID) for the storage nodes containing
        the legacy security event data for the user with the given UID.
        `REST Documentation <https://console.us.code42.com/swagger/#/Feature/getStorageNode>`__

        Args:
            user_uid (str): A UID for the user to get plan storage information for.

        Returns:
            list[:class:`py42.modules.securitydata.PlanStorageInfo`]
        """
        locations = None
        try:
            response = self._security_client.get_security_event_locations(user_uid)
            locations = response[u"securityPlanLocationsByDestination"]
        except HTTPError as err:
            if err.response.status_code == 404:
                pass
            else:
                raise_py42_error(err)

        if locations:
            plan_destination_map = _get_plan_destination_map(locations)
            selected_plan_infos = self._get_plan_storage_infos(plan_destination_map)
            if not selected_plan_infos:
                raise Py42SecurityPlanConnectionError(
                    u"Could not establish a connection to retrieve "
                    u"security events for user {0}".format(user_uid)
                )

            return selected_plan_infos

    def get_all_plan_security_events(
        self,
        plan_storage_info,
        cursor=None,
        include_files=True,
        event_types=None,
        min_timestamp=None,
        max_timestamp=None,
    ):
        """Gets legacy endpoint monitoring events. This does not retrieve cloud events.
        `Support Article <https://support.code42.com/Administrator/6/Configuring/Endpoint_monitoring>`__

        Args:
            plan_storage_info (:class:`py42.sdk.modules.securitydata.PlanStorageInfo`):
                Information about storage nodes for a plan to get security events for.
            cursor (str, optional): A cursor position for only getting events you did not
                previously get. Defaults to None.
            include_files (bool, optional): Whether to include the files related to the security
                events. Defaults to None.
            event_types: (str, optional): A comma-separated list of event types to filter by.

                    Available options are:
                        - ``DEVICE_APPEARED``
                        - ``DEVICE_DISAPPEARED``
                        - ``DEVICE_FILE_ACTIVITY``
                        - ``PERSONAL_CLOUD_FILE_ACTIVITY``
                        - ``RESTORE_JOB``
                        - ``RESTORE_FILE``
                        - ``FILE_OPENED``
                        - ``RULE_MATCH``
                        - ``DEVICE_SCAN_RESULT``
                        - ``PERSONAL_CLOUD_SCAN_RESULT``

                    Defaults to None.
            min_timestamp (float, optional): A POSIX timestamp representing the lower bound of the
                date range of events to get. Defaults to None.
            max_timestamp (float, optional): A POSIX timestamp representing the upper bound of the
                date range of events to get. Defaults to None.

        Returns:
            generator: An object that iterates over :class:`py42.response.Py42Response` objects
            that each contain a page of events.
        """
        return self._get_security_detection_events(
            plan_storage_info, cursor, include_files, event_types, min_timestamp, max_timestamp
        )

    def get_all_user_security_events(
        self,
        user_uid,
        cursor=None,
        include_files=True,
        event_types=None,
        min_timestamp=None,
        max_timestamp=None,
    ):
        """Gets legacy security events for the user with the given UID.

        Args:
            user_uid (str): The UID for the user to get security events for.
            cursor (str, optional): A cursor position for only getting events you did not
                previously get. Defaults to None.
            include_files (bool, optional): Whether to include the files related to the security
                events. Defaults to None.
            event_types: (str, optional): A comma-separated list of event types to filter by.

                    Available options are:
                        - ``DEVICE_APPEARED``
                        - ``DEVICE_DISAPPEARED``
                        - ``DEVICE_FILE_ACTIVITY``
                        - ``PERSONAL_CLOUD_FILE_ACTIVITY``
                        - ``RESTORE_JOB``
                        - ``RESTORE_FILE``
                        - ``FILE_OPENED``
                        - ``RULE_MATCH``
                        - ``DEVICE_SCAN_RESULT``
                        - ``PERSONAL_CLOUD_SCAN_RESULT``

                    Defaults to None.
            min_timestamp (float, optional): A POSIX timestamp representing the lower bound of the
                date range of events to get. Defaults to None.
            max_timestamp (float, optional): A POSIX timestamp representing the upper bound of the
                date range of events to get. Defaults to None.

        Returns:
            generator: An object that iterates over :class:`py42.response.Py42Response` objects
            that each contain a page of events.
        """
        security_plan_storage_infos = self.get_security_plan_storage_info_list(user_uid)
        return self._get_security_detection_events(
            security_plan_storage_infos,
            cursor,
            include_files,
            event_types,
            min_timestamp,
            max_timestamp,
        )

    def search_file_events(self, query):
        """Searches for file events.
        `REST Documentation <https://support.code42.com/Administrator/Cloud/Monitoring_and_managing/Forensic_File_Search_API>`__

        Args:
            query (:class:`py42.sdk.queries.fileevents.file_event_query.FileEventQuery`): Also
                accepts a raw JSON str.

        Returns:
            :class:`py42.response.Py42Response`: A response containing the first 10,000
            events.
        """
        file_event_client = self._microservices_client_factory.get_file_event_client()
        return file_event_client.search(query)

    def _get_plan_storage_infos(self, plan_destination_map):
        plan_infos = []
        for plan_uid in plan_destination_map:
            destinations = plan_destination_map[plan_uid]
            storage_info = self._get_storage_info_for_plan(plan_uid, destinations)
            if storage_info:
                plan_infos.append(storage_info)

        return plan_infos

    def _get_storage_info_for_plan(self, plan_uid, destinations):
        for destination in destinations:
            # try to connect to every storage node for this plan until one works
            plan_storage_info = self._get_storage_info_for_plan_destination(plan_uid, destination)
            if plan_storage_info:
                return plan_storage_info

    def _get_storage_info_for_plan_destination(self, plan_uid, destination):
        try:
            destination_guid = destination[u"destinationGuid"]
            node_guid = destination[u"nodeGuid"]
            plan_storage_info = PlanStorageInfo(plan_uid, destination_guid, node_guid)
            self._try_get_security_detection_event_client(plan_storage_info)
            return plan_storage_info
        except HTTPError:
            #  This function is called in a loop until we get a result that is not None.
            #  If all return None, then the calling function raises Py42SecurityPlanConnectionError.
            pass

    def _try_get_security_detection_event_client(self, plan_storage_info):
        # check if we have already created and stored this client
        client = self._client_cache.get(plan_storage_info.node_guid)

        # otherwise, create it
        if client is None:
            client = self._storage_client_factory.from_plan_info(
                plan_storage_info.plan_uid, plan_storage_info.destination_guid
            ).securitydata

            # store this client via its guid so that we don't have to call StorageAuthToken
            # just to determine what storage client to use
            with self._client_cache_lock:
                self._client_cache.update({plan_storage_info.node_guid: client})

        return client

    def _get_security_detection_events(
        self, plan_storage_infos, cursor, include_files, event_types, min_timestamp, max_timestamp
    ):
        if type(plan_storage_infos) is not list:
            plan_storage_infos = [plan_storage_infos]

        # get the storage node client for each plan
        for plan_storage_info in plan_storage_infos:
            client = self._try_get_security_detection_event_client(plan_storage_info)
            started = False

            # get all pages of events for this plan
            while cursor or not started:
                started = True
                response = client.get_plan_security_events(
                    plan_storage_info.plan_uid,
                    cursor=cursor,
                    include_files=include_files,
                    event_types=event_types,
                    min_timestamp=min_timestamp,
                    max_timestamp=max_timestamp,
                )

                if response.text:
                    # we use json.loads here because the cursor prop doesn't appear
                    # on responses that have no results
                    cursor = json.loads(response.text).get(u"cursor")
                    # if there are no results, we don't get a cursor and have reached the end
                    if cursor:
                        yield response, cursor


def _get_plan_destination_map(locations_list):
    plan_destination_map = {}
    for plans in _get_destinations_in_locations_list(locations_list):
        for plan_uid in plans:
            plan_destination_map[plan_uid] = plans[plan_uid]
    return plan_destination_map


def _get_destinations_in_locations_list(locations_list):
    for destination in locations_list:
        for node in destination[u"securityPlanLocationsByNode"]:
            yield _get_plans_in_node(destination, node)


def _get_plans_in_node(destination, node):
    return {
        plan_uid: [
            {u"destinationGuid": destination[u"destinationGuid"], u"nodeGuid": node[u"nodeGuid"]}
        ]
        for plan_uid in node[u"securityPlanUids"]
    }


class PlanStorageInfo(object):
    def __init__(self, plan_uid, destination_guid, node_guid):
        self._plan_uid = plan_uid
        self._destination_uid = destination_guid
        self._node_guid = node_guid

    @property
    def plan_uid(self):
        """A UID for a storage plan."""
        return self._plan_uid

    @property
    def destination_guid(self):
        """A GUID for the destination containing the storage archive."""
        return self._destination_uid

    @property
    def node_guid(self):
        """The GUID for the storage node containing the archive."""
        return self._node_guid
